SUBROUTINE POS(&
 & KPROMA,KPROMVC,KPROMDLR,KST,KND,&
 & KOPLEV,KGFL,KFLDSPP,KFLDAUX,KFPLEV,KFLEVG,KPPM,CDCONF,YDIN,YDGFL,&
 & PVAH,PVBH,PRCORI,PGM,POROG,POROGL,POROGM,PXORO,&
 & PXLEV,PTSI,PXTSO,PEV, &
 & PU,PUL,PDIV,PV,PVL,PVOR,PT,PTL,PTM,PSPD,PSPDL,PSPDM,PSVD,PNHX,&
 & PSP,PSPL,PSPM,&
 & PGFL,PEXTRA,PEXRAD,PRMCUFGP,PWW,PGPP,PAUX)

!**** *POS* - post-processing

!     PURPOSE.
!     -------
!           Post-processing calculations.

!**   INTERFACE.
!     ---------
!        *CALL* *POS(...)

!        EXPLICIT ARGUMENTS :
!        ------------------
!        * INPUT:
!        KPROMA    : horizontal dimension.
!        KPROMVC   : horizontal dimension for LVERCOR option arrays.
!        KPROMDLR  : horizontal dimension for LRWSDLR option arrays.
!        KST       : first point to post-process 
!        KND       : last point to post-process 
!        KOPLEV    : number of post-processing levels
!        KGFL      : number of GFL fields in PGFL
!        KFLDSPP   : number of fields to be spectrally fitted
!        KFLDAUX   : number of fields to remain grid point
!        KFPLEV    : number of output vertical levels
!        KFLEVG    : number of model vertical levels (NFLEVG)
!        KPPM      : number of interpolation methods in post-processing (NPPM)
!        CDCONF    : configuration of work
!                    'B' <=> Pressure                 level pp.
!                    'H' <=> Height (above orography) level pp.
!                    'K' <=> Temperature              level pp.
!                    'T' <=> Potential temperature    level pp.
!                    'V' <=> Potential vorticity      level pp.
!                    'M' <=> Model                    level pp. 
!                    'S' <=> Eta                      level pp.
!        YDIN      : pointers allowing to retrieve individual fields in PGFL
!        YDGFL     : says if GFL is active or not
!        PVAH      : output vertical function *A*, for APACHE
!        PVBH      : output vertical function *B*, for APACHE
!        PRCORI    : coriolis parameter
!        PGM       : map factor
!        POROG     : orography
!        POROGL    : zonal gradient of orography
!        POROGM    : meridian gradient of orography
!        PXORO     : external orography ("interpolated"), for APACHE
!        PXLEV     : array containing the pp levels (in P,H,PV,TH or s)
!        PTSI      : input surface temperature
!        PXTSO     : output surface temperature, for APACHE
!        PEV       : <eta dot * dp/deta> on top of the atmosphere
!        PU        : U-wind
!        PUL       : zonal gradient of U-wind
!        PDIV      : divergence
!        PV        : V-wind
!        PVL       : zonal gradient of V-wind
!        PVOR      : vorticity
!        PT        : temperature
!        PTL       : zonal gradient of temperature
!        PTM       : meridian gradient of temperature
!        PSPD      : pressure departure variable if LFPART2=F;
!                    pressure departure "pre-prehyd" if LFPART2=T.
!        PSPDL     : zonal gradient of pressure departure var (LFPART2=F only)
!        PSPDM     : merid gradient of pressure departure var (LFPART2=F only)
!        PSVD      : vertical divergence variable if LFPART2=F;
!                    -G*(Delta w) if LFPART2=T.
!        PNHX      : term NHX, used only if
!                    LSLAG.AND.LNHDYN.AND.(NVDVAR==4).AND.(ND4SYS==2)
!        PSP       : surface pressure variable
!                    (contains log(presims) if lrwsdlr=T)
!        PSPL      : zonal gradient of surface pressure variable
!        PSPM      : meridian gradient of surface pressure variable
!        PGFL      : GFL fields
!        PEXTRA    : extra input fields
!        PEXRAD    : extra input fields from radiation code
!        PRMCUFGP  : CUF input fields
!     
!        * OUTPUT (auxiliary)
!        PWW       : geometric vertical velocity - so it is available in PHYMFPOS

!        * OUTPUT (post-processed fields):
!        PGPP      : fields to be spectrally fitted
!        PAUX      : fields to remain gridpoint

!     EXTERNALS.
!     ---------

! Author
! ------
!   ORIGINAL : 88-02-04

! Modifications
! -------------
!   R. El Khatib : 02-21-20 Fullpos B-level distribution + remove IO scheme
!   F. Vana      : 02-07-03 Virtual potential temperature
!   R. El Khatib : 02-09-04 Iso -10 Celsius
!   R. El Khatib : 03-01-06 Computations savings and protections
!   R. El Khatib : 03-08-12 gfl
!   R. El Khatib : 03-04-17 Fullpos improvemnts
!   01-Oct-2003 M. Hamrud  CY28 Cleaning
!   20-Jun-2004 J. Masek   (NPDVAR, NVDVAR) = (2, 3)
!   R. El Khatib : 04-04-14 Optimization
!   14-Mar-2005 R. Engelen    Greenhouse Gase
!   14-Mar-2005 A. Untch    Aerosols
!   11-Apr-2005 J Flemming Aerosol replaced with GEMS GRG 
!   Y. Bouteloup : 28-Jan-2005 Add PR (precipitation)
!   K. Yessad    : 04-01-24 EVERDIA renamed into GPVERDIA
!   G. Hello     : 05-01-27 snow, rain, graupel, tke
!   K. Yessad    : 04-01-25 obsolete EGPHI replaced by GPGRXYB+GPGRGEO
!   R. El Ouaraini & Ryad El Khatib : 05-07-28 Full-pos CUF fields
!   M. Bellus    : 29-Sep-2006 prognostic convection and MAC scheme (ALARO-0)
!   S. Serrar    : 07-09-06 include tracers used for diagnostics only (GEMS)
!   G. Hello     : 07-02 thetav
!   S. Serrar    : 22-03-2007 post-process new GFL ERA40 fields
!   S. Serrar    : 17-07-2007 two new GFL fields methane related
!   K. Yessad    : 10-10-2007 (provisional) adaptations to VFE for NH variables
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme + etadot
!   S. Serrar   02-05-2008 test on LERA40 removed
!   K. Yessad (Feb 2008): paragraph renumbering, specific paragraphes for GFL 
!   K. Yessad (Aug 2008): (partial) more concise rewriting.
!   JJMorcrette : 20090217 PP of aerosol and UV processor outputs
!   K. Yessad (Aug 2009): rewrite vertical interpolator in post-processing.
!   Y. Bouteloup  : 14-07-2009 Add radiative cloud water and ice (YIRAD and YLRAD)
!   K. Yessad (Jan 2011): introduce INTDYN_MOD structures.
!   H. Hersbach   : 01-04-2011 auxiliary diagnostic radiation fields
!   K. Yessad (Dec 2011): various contributions.
!   J. Cedilnik   : 18-06-2013 Add geometrical vertical velocity in output
!   J. Cedilnik and C. Wittman : 31-08-2013 lighting diagnostics and vertical temp gradient
! End Modifications
!------------------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE PARFPOS  , ONLY : JPOSFSU  ,JPOSVX2
USE YOMDIM   , ONLY : LVOR     ,LUVDER  
USE YOMCT0   , ONLY : LFPART2  ,LNHDYN   ,LSPRT    ,LVERCOR  ,LSLAG
USE YOMDYNA  , ONLY : NVDVAR   ,ND4SYS   ,LNH_GEOGW,LRWSDLR  ,LRWSDLR2
USE YOMGEM   , ONLY : YRVAB    ,YRVETA
USE YOMCST   , ONLY : RG       ,RATM     ,RD       ,RV       ,RCPD     ,&
 & RKAPPA   ,RTT  
USE YOMSTA   , ONLY : NLEXTRAP
USE YOMCVER  , ONLY : LVERTFE
USE YOMFPC   , ONLY : LFPLOSP  ,LFPRH100 ,CFPFMT   ,FPBL     ,LFPQ     ,&
 & FPRHMIN  ,FPRHMAX
USE YOMFPG   , ONLY : LFPOSVER
USE YOMVPOS  , ONLY : QFPTYPE
USE YOMAFN   , ONLY : TFP_ABS  ,TFP_DIV  ,TFP_ETH  ,TFP_FIS   ,&
 & TFP_FOL  ,TFP_FSU  ,TFP_FUA  ,TFP_GM   ,TFP_KHI  ,TFP_LNSP ,&
 & TFP_MG   ,TFP_MOCO ,TFP_MSL  ,TFP_PCAO ,TFP_PD   ,TFP_PJET ,&
 & TFP_PSI  ,TFP_PV   ,TFP_P    ,TFP_HU   ,TFP_SHD  ,TFP_SP   ,&
 & TFP_STD  ,TFP_TCAO ,TFP_TH   ,TFP_THPW ,TFP_T    ,TFP_UJET ,&
 & TFP_U    ,TFP_VD   ,TFP_VJET ,TFP_VOR  ,TFP_VV   ,TFP_VW   ,&
 & TFP_V    ,TFP_WND  ,TFP_TWV  ,TFP_WWS  ,TFP_Z    ,TFP_PTB  ,&
 & TFP_HTB  ,TFP_HTPW ,TFP_IET  ,TFP_RHO  ,TFP_QNH  ,TFP_VTG  ,&
 & TFP_CUF1 ,TFP_CUF2 ,TFP_CUF3 ,TFP_CUF4 ,TFP_CUF5 ,TFP_SRE  ,&
 & TFP_THV  ,TFP_ETAD, TFP_FLASH
USE YOMMCUF  , ONLY : NCUFNR
USE SURFACE_FIELDS_MIX  , ONLY : YSD_XA,YSD_XAD, YSD_XR,YSD_XRD
USE YOM_YGFL , ONLY : NGFL_EXT, NAERO, NGHG, NGRG, NTRAC, NERA40,&
 &  NNOGW, NAEROUT, NUVP
USE TYPE_GFLFLDS, ONLY: TYPE_IGFLFLD,TYPE_IGFLFLDD,TYPE_LGFLFLDD
USE PAR_RDLR , ONLY : JP_DIMR0
USE INTDYN_MOD,ONLY : YYTVCPP  ,YYTHWPP  ,YYTCTYPP ,YYTXYBDERPP, YYTXYBPP

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMVC
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMDLR
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KOPLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGFL
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDSPP 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDAUX 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFPLEV 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLEVG
INTEGER(KIND=JPIM),INTENT(IN)    :: KPPM
CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
TYPE(TYPE_IGFLFLDD),INTENT(IN)   :: YDIN
TYPE(TYPE_LGFLFLDD),INTENT(IN)   :: YDGFL
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVAH(0:KFPLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVBH(0:KFPLEV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRCORI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGL(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROGM(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXORO(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXLEV(KOPLEV)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTSI(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXTSO(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEV(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PU(KPROMA,KFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUL(KPROMA,KFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PDIV(KPROMA,KFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PV(KPROMA,KFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVL(KPROMA,KFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVOR(KPROMA,KFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PT(KPROMA,KFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTL(KPROMA,KFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTM(KPROMA,KFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPD(KPROMA,KFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPDL(KPROMA,KFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPDM(KPROMA,KFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSVD(KPROMA,KFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PNHX(KPROMA,KFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSP(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPL(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFL(KPROMA,KFLEVG,KGFL)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEXTRA(KPROMA,YSD_XAD%NLEVS,YSD_XAD%NDIM) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PEXRAD(KPROMA,YSD_XRD%NLEVS,YSD_XRD%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PRMCUFGP(KPROMA,MAX(1,NCUFNR)) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGPP(KPROMA,KFLDSPP) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PAUX(KPROMA,KFLDAUX)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PWW(KPROMA,KFLEVG) 

!     ------------------------------------------------------------------

! ==== REAL COMPUTED IN PARTS 1.2.1, 1.2.2, 1.2.3  ===================
REAL(KIND=JPRB) :: ZOROGL(KPROMA),ZOROGM(KPROMA),ZGMAPP(KPROMA)
REAL(KIND=JPRB),ALLOCATABLE :: ZGFLT0(:,:,:)
REAL(KIND=JPRB) :: ZQT0L(KPROMA,KFLEVG),ZQT0M(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZGRHLT0(KPROMA,0:KFLEVG)
REAL(KIND=JPRB) :: ZUT0(KPROMA,0:KFLEVG),ZVT0(KPROMA,0:KFLEVG)
REAL(KIND=JPRB) :: ZUT0L(KPROMA,0:KFLEVG),ZVT0L(KPROMA,0:KFLEVG)
REAL(KIND=JPRB) :: ZDIVT0(KPROMA,0:KFLEVG),ZVORT0(KPROMA,0:KFLEVG)
REAL(KIND=JPRB) :: ZTT0(KPROMA,0:KFLEVG)
REAL(KIND=JPRB) :: ZTT0L(KPROMA,KFLEVG),ZTT0M(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZPDT0(KPROMA,0:KFLEVG)
REAL(KIND=JPRB) :: ZPDT0L(KPROMA,KFLEVG),ZPDT0M(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZSP0L(KPROMA),ZSP0M(KPROMA)

! ==== REAL COMPUTED IN PART 2.1  ====================================
REAL(KIND=JPRB) :: ZR(KPROMA,KFLEVG),ZCP(KPROMA,KFLEVG),ZKAP(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZPRESH(KPROMA,0:KFLEVG),ZPRESF(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZXYB(KPROMA,KFLEVG,YYTXYBPP%NDIM)
REAL(KIND=JPRB) :: ZRPRESF(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZXYBDER(KPROMA,KFLEVG,YYTXYBDERPP%NDIM)
REAL(KIND=JPRB) :: ZRT(KPROMA,KFLEVG),ZRTL(KPROMA,KFLEVG),ZRTM(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZPDEP(KPROMA,0:KFLEVG),ZNHPREF(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZNHPPI(KPROMA,KFLEVG),ZRNHPPI(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZRRED(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZUSL_NHPREL(KPROMA,KFLEVG),ZUSL_NHPREM(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZUSL_LNNHPREFL(KPROMA,KFLEVG),ZUSL_LNNHPREFM(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZQCHAL(KPROMA,KFLEVG),ZQCHAM(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZGEOPH(KPROMA,0:KFLEVG),ZGEOPF(KPROMA,0:KFLEVG)
REAL(KIND=JPRB) :: ZRDLR(KPROMDLR,0:KFLEVG,JP_DIMR0)
REAL(KIND=JPRB) :: ZGPFL(KPROMA,KFLEVG),ZGPFM(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZGPHL(KPROMA,0:KFLEVG),ZGPHM(KPROMA,0:KFLEVG)
REAL(KIND=JPRB) :: ZVCRS(KPROMVC,0:KFLEVG,YYTVCPP%NDIM)
REAL(KIND=JPRB) :: ZETADOT(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZCTY(KPROMA,0:KFLEVG,YYTCTYPP%NDIM)
REAL(KIND=JPRB) :: ZUVH(KPROMA,0:KFLEVG,YYTHWPP%NDIM)
REAL(KIND=JPRB) :: ZUS(KPROMA),ZVS(KPROMA)
REAL(KIND=JPRB) :: ZNHX(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZDVER(KPROMA,KFLEVG),ZWW(KPROMA,0:KFLEVG)
REAL(KIND=JPRB) :: ZMGDW(KPROMA,KFLEVG),ZGDW(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZGW(KPROMA,0:KFLEVG),ZGWF(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZPRESIMF(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZR2(KPROMA,KFLEVG)

! ==== REAL COMPUTED IN PART 2.2  ====================================
REAL(KIND=JPRB) :: ZXGEOPH(KPROMA,0:KFLEVG),ZUSL_GEOPF(KPROMA,KFLEVG)
REAL(KIND=JPRB) :: ZES(KPROMA,KFLEVG),ZRHF(KPROMA,0:KFLEVG)
REAL(KIND=JPRB) :: ZTOUP(KPROMA,0:KFLEVG),ZTETA(KPROMA,0:KFLEVG)
REAL(KIND=JPRB) :: ZSTDF(KPROMA,0:KFLEVG),ZSHDF(KPROMA,0:KFLEVG)
REAL(KIND=JPRB) :: ZEPTH(KPROMA,0:KFLEVG),ZIBET(KPROMA,0:KFLEVG)
REAL(KIND=JPRB) :: ZSIMR(KPROMA,0:KFLEVG),ZVIRT(KPROMA,0:KFLEVG)

! vertical temperature gradient, by JC
REAL(KIND=JPRB) :: ZVTGH(KPROMA,0:KFLEVG),ZVTGF(KPROMA,KFLEVG)


! ==== REAL COMPUTED IN PART 3    ====================================
REAL(KIND=JPRB) :: ZTSTAR(KPROMA),ZT0(KPROMA)
REAL(KIND=JPRB) :: ZPRES(KPROMA,KOPLEV),ZLNPRES(KPROMA,KOPLEV)
REAL(KIND=JPRB) :: ZGEOP(KPROMA,KOPLEV)
REAL(KIND=JPRB) :: ZPXPD(KPROMA,0:KFLEVG,KPPM),ZPXP(KPROMA,0:KFLEVG,KPPM)
REAL(KIND=JPRB) :: ZSTTF(KPROMA,0:KFLEVG),ZSTZF(KPROMA,0:KFLEVG)

! ==== REAL COMPUTED IN PART 4    ====================================
REAL(KIND=JPRB),ALLOCATABLE :: ZEXTRA(:,:)
REAL(KIND=JPRB),ALLOCATABLE :: ZEXRAD(:,:)
REAL(KIND=JPRB) :: ZXLEV(KOPLEV),ZTPW(KPROMA,KFLEVG)

! ==== LOCAL POST-PROCESSED FIELDS  ==================================
REAL(KIND=JPRB),ALLOCATABLE :: ZPPGFL(:,:,:)   ! GFL
REAL(KIND=JPRB) :: ZPPU(KPROMA,KOPLEV)         ! U
REAL(KIND=JPRB) :: ZPPV(KPROMA,KOPLEV)         ! V
REAL(KIND=JPRB) :: ZPPT(KPROMA,KOPLEV)         ! T
REAL(KIND=JPRB) :: ZPPUAP(KPROMA,KOPLEV)       ! upper air prehyd
REAL(KIND=JPRB) :: ZPPSP(KPROMA)               ! prehyds
REAL(KIND=JPRB) :: ZPPLNSP(KPROMA)             ! log(prehyds)
REAL(KIND=JPRB) :: ZPPPDEP(KPROMA,KOPLEV)      ! pressure departure
REAL(KIND=JPRB) :: ZPPWVEL(KPROMA,KOPLEV)      ! w
REAL(KIND=JPRB) :: ZPPMGDW(KPROMA,KOPLEV)      ! -G*Delta w
REAL(KIND=JPRB) :: ZPPPHI(KPROMA,KOPLEV)       ! geopotential height
REAL(KIND=JPRB) :: ZPPMG(KPROMA,KOPLEV)        ! Montgomery potential
REAL(KIND=JPRB) :: ZPPHRL(KPROMA,KOPLEV)       ! relative humidity
REAL(KIND=JPRB) :: ZPPVVE(KPROMA,KOPLEV)       ! omega=Dprehyd/Dt
REAL(KIND=JPRB) :: ZPPEVEL(KPROMA,KOPLEV)      ! etadot
REAL(KIND=JPRB) :: ZPPMSLP(KPROMA)             ! MSL pressure
REAL(KIND=JPRB) :: ZPPCUF(KPROMA,NCUFNR)       ! CUF
REAL(KIND=JPRB) :: ZPPPOTV(KPROMA,KOPLEV)      ! potential vorticity
REAL(KIND=JPRB) :: ZPPTETA(KPROMA,KOPLEV)      ! potential temperature
REAL(KIND=JPRB) :: ZPPEPTH(KPROMA,KOPLEV)      ! equivalent potential temp.
REAL(KIND=JPRB) :: ZPPSTDF(KPROMA,KOPLEV)      ! stretching deformation
REAL(KIND=JPRB) :: ZPPSHDF(KPROMA,KOPLEV)      ! shearing deformation
REAL(KIND=JPRB) :: ZPPTWV(KPROMA)              ! total water vapour
REAL(KIND=JPRB) :: ZPPFOLDIN(KPROMA)           ! tropopause folding indicator
REAL(KIND=JPRB) :: ZPPIBET(KPROMA,KOPLEV)      ! isobaric equivalent temperature
REAL(KIND=JPRB) :: ZPPSIMR(KPROMA,KOPLEV)      ! simulated reflectivities
REAL(KIND=JPRB) :: ZPPVIRT(KPROMA,KOPLEV)      ! virtual potential temperature
REAL(KIND=JPRB) :: ZPPTHPW(KPROMA,KOPLEV)      ! theta'w
REAL(KIND=JPRB) :: ZPPWIND(KPROMA,KOPLEV)      ! wind velocity
REAL(KIND=JPRB) :: ZPPDIV(KPROMA,KOPLEV)       ! horizontal divergence
REAL(KIND=JPRB) :: ZPPVOR(KPROMA,KOPLEV)       ! vorticity
REAL(KIND=JPRB) :: ZPPABS(KPROMA,KOPLEV)       ! absolute vorticity
REAL(KIND=JPRB) :: ZPPPTB(KPROMA,KOPLEV)       ! pressure of iso-T
REAL(KIND=JPRB) :: ZPPHTB(KPROMA,KOPLEV)       ! height at iso-T
REAL(KIND=JPRB) :: ZPPPICAO(KPROMA)            ! ICAO tropopause pressure
REAL(KIND=JPRB) :: ZPPTICAO(KPROMA)            ! ICAO tropopause temperature
REAL(KIND=JPRB) :: ZPPUJET(KPROMA)             ! max U at the jet
REAL(KIND=JPRB) :: ZPPVJET(KPROMA)             ! max V at the jet
REAL(KIND=JPRB) :: ZPPPJET(KPROMA)             ! pressure at the max wind level
REAL(KIND=JPRB) :: ZPPHTPW(KPROMA)             ! height at iso-Theta'w=0C
REAL(KIND=JPRB) :: ZPPMOCO(KPROMA)             ! moisture convergence
REAL(KIND=JPRB) :: ZPPRHO(KPROMA,KOPLEV)       ! humid density
REAL(KIND=JPRB) :: ZPPQNH(KPROMA)              ! QNH pressure
REAL(KIND=JPRB),ALLOCATABLE :: ZPPFUA(:,:,:)   ! free upper air fields
REAL(KIND=JPRB),ALLOCATABLE :: ZPPFSU(:,:)     ! free surface fields
REAL(KIND=JPRB) :: ZPPIN(KPROMA,KOPLEV),ZPPIN1(KPROMA) ! work arrays
! Z(X)H: (X) post-processed at half-levels of pp. 
!        Z(X)H(.,JLEV,0) : half level above full level JLEV
!        Z(X)H(.,JLEV,1) : half level below full level JLEV
REAL(KIND=JPRB) :: ZPPWVELH(KPROMA,KOPLEV,0:1)  ! w
REAL(KIND=JPRB) :: ZPH     (KPROMA,KOPLEV,0:1)  ! prehyd
REAL(KIND=JPRB) :: ZPPR(KPROMA,KOPLEV)
REAL(KIND=JPRB) :: ZPPRT(KPROMA,KOPLEV)
REAL(KIND=JPRB) :: ZPPVTG(KPROMA,KOPLEV)

! ==== OTHER REAL VARIABLES ==========================================
REAL(KIND=JPRB) :: ZWORK(KPROMA),ZLNWORK(KPROMA)
REAL(KIND=JPRB) :: Z1SRG, ZDUMARR(1), ZTROPO, ZXTEMP, ZRHBNDS(2)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

! ==== INTEGER VARIABLES =============================================
INTEGER(KIND=JPIM) :: IHILEV, ILEVM, ILOLEV, ILOC, ILOCU, ILOCV, IPTR, ILPT, I
INTEGER(KIND=JPIM) :: IIPROMVC
INTEGER(KIND=JPIM) :: JH, JL, JLEV, JLEVP, JROF, JVAR, JVEXTR, JCUFNR
INTEGER(KIND=JPIM) :: IPOL2 ! input to POAERO
INTEGER(KIND=JPIM) :: ISLCT1, ISLCT2 ! input to PP2DINT
INTEGER(KIND=JPIM) :: IPPGFL, INFUA, INFSU ! last dimension for ZPP... arrays
INTEGER(KIND=JPIM),ALLOCATABLE :: IGFLCOD(:), IDIN_GFL(:)
INTEGER(KIND=JPIM) :: ICUFCOD(NCUFNR)
INTEGER(KIND=JPIM) :: ILEVB(KPROMA,KOPLEV,KPPM)
INTEGER(KIND=JPIM) :: ILEVBH(KPROMA,KOPLEV,KPPM,0:1)
INTEGER(KIND=JPIM) :: IPTRLEV(KOPLEV) ! IPTRLEV : absolute indexes of levels
TYPE(TYPE_IGFLFLD) :: YLPPIC

! ==== LOGICAL LL[X] VARIABLES FOR INPUT MEMORY TRANSFER =============
LOGICAL :: LLUVDER, LLDIV0, LLVOR0, LLTDER, LLQDER, LLPVO
LOGICAL,ALLOCATABLE :: LLIN_GFL(:) ! Input GFL

! ==== LOGICAL LL[X] VARIABLES FOR PP ================================
LOGICAL,ALLOCATABLE :: LL_GFL(:) ! GFL
LOGICAL,ALLOCATABLE :: LLAP_GFL(:) ! GFL (for APACHE)
LOGICAL :: LL_PPUV    ! Wind vector (U,V)
LOGICAL :: LL_T       ! temperature
LOGICAL :: LL_P       ! Pressure
LOGICAL :: LL_SP      ! surface pressure
LOGICAL :: LL_LNSP    ! log. surface pressure
LOGICAL :: LL_PD      ! n.-h. pressure departure 
LOGICAL :: LL_VW      ! true vertical velocity w
LOGICAL :: LL_WWS     ! surface vertical velocity
LOGICAL :: LL_VD      ! vertical divergence variable -G*(Delta w)
LOGICAL :: LL_Z       ! geopotential height
LOGICAL :: LL_MG      ! Montgomery potential
LOGICAL :: LL_HU      ! relative humidity
LOGICAL :: LL_VV      ! vertical velocity omega
LOGICAL :: LL_ETAD    ! vertical velocity etadot
LOGICAL :: LL_MSL     ! mean sea level pressure
LOGICAL :: LL_CUF(NCUFNR) ! Filtred ln(Ps) for monitoring Coupling Updates Frequency
LOGICAL :: LL_PV      ! Potential Vorticity
LOGICAL :: LL_TH      ! Potential temperature
LOGICAL :: LL_VTG     ! Vertical temperature gradient
LOGICAL :: LL_ETH     ! Equivalent pot. temperature
LOGICAL :: LL_STD     ! Stretching Deformation
LOGICAL :: LL_SHD     ! Shearing Deformation
LOGICAL :: LL_TWV     ! Total water vapour
LOGICAL :: LL_FOL     ! tropopause folding ind.
LOGICAL :: LL_IET     ! Isobaric equivalent temperature
LOGICAL :: LL_SRE     ! Simulated reflectivity
LOGICAL :: LL_THV     ! virtual t or theta
LOGICAL :: LL_THPW    ! theta'w
LOGICAL :: LL_WND     ! wind velocity
LOGICAL :: LL_DIV     ! Divergence
LOGICAL :: LL_VOR     ! Relative Vorticity
LOGICAL :: LL_ABS     ! Absolute Vorticity
LOGICAL :: LL_PCAO    ! ICAO tropopause pressure
LOGICAL :: LL_TCAO    ! ICAO tropopause temperature
LOGICAL :: LL_UJET    ! ICAO U component of the jet
LOGICAL :: LL_VJET    ! ICAO V component of the jet
LOGICAL :: LL_PJET    ! ICAO pressure at jet level
LOGICAL :: LL_PTB    ! Pressure of iso-T
LOGICAL :: LL_HTB    ! height of iso-T
LOGICAL :: LL_HTPW    ! height of iso-T'w=0 Kelvin, from bottom
LOGICAL :: LL_MOCO    ! Moisture convergence
LOGICAL :: LL_RHO     ! humid density
LOGICAL :: LL_QNH     ! qnh pressure
LOGICAL :: LL_FLASH   ! lightening diagnostics
LOGICAL :: LL_FUA(JPOSVX2) ! Free upper air fields
LOGICAL :: LL_FSU(JPOSFSU) ! Free surface fields

! ==== OTHER LOGICAL VARIABLES =======================================
LOGICAL :: LL,LLAERO,LLVERCOR,LLTOP,LLEXTR,LLDYN(4),LLAP,LLPD,LLGEO,LLCTY,LLWW
LOGICAL :: LLGWF,LLGDWI,LLUNIT,LLVFE,LL_TSTAR
LOGICAL :: LLBELO(KPROMA,KOPLEV), LLBLOW(KOPLEV)
LOGICAL :: LLBELS(KPROMA,KOPLEV), LLBLES(KOPLEV)
LOGICAL :: LLBELOH(KPROMA,KOPLEV,0:1), LLBLOWH(KOPLEV,0:1)
LOGICAL :: LLBELSH(KPROMA,KOPLEV,0:1), LLBLESH(KOPLEV,0:1)

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "apache.intfb.h"
#include "ctstar.intfb.h"
#include "fpps.intfb.h"
#include "gnhdlr.intfb.h"
#include "gnhdlrb.intfb.h"
#include "gnhgrdlr.intfb.h"
#include "gnhgrpre.intfb.h"
#include "gnhpre.intfb.h"
#include "gpcty.intfb.h"
#include "gpept.intfb.h"
#include "gpgeo.intfb.h"
#include "gpgrgeo.intfb.h"
#include "gpvcrs.intfb.h"
#include "gpxx.intfb.h"
#include "gphlwi.intfb.h"
#include "gphluv.intfb.h"
#include "gpgrxyb.intfb.h"
#include "gpgw.intfb.h"
#include "gpiet.intfb.h"
#include "gphpre.intfb.h"
#include "gpprs0d.intfb.h"
#include "gppvo.intfb.h"
#include "gprcp.intfb.h"
#include "gprh.intfb.h"
#include "gprt.intfb.h"
#include "gpuvs.intfb.h"
#include "poaero.intfb.h"
#include "pos_prepgfl.intfb.h"
#include "pp2dint.intfb.h"
#include "ppflev.intfb.h"
#include "ppgeop.intfb.h"
#include "ppinit.intfb.h"
#include "ppleta.intfb.h"
#include "ppltemp.intfb.h"
#include "pplteta.intfb.h"
#include "ppltp.intfb.h"
#include "pppmer.intfb.h"
#include "ppq.intfb.h"
#include "ppt.intfb.h"
#include "ppthpw.intfb.h"
#include "ppuv.intfb.h"
#include "ppvvel.intfb.h"
#include "ppwetpoint.intfb.h"
#include "ppsta.intfb.h"
#include "ppcvirt.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('POS',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*       1.    INITIALIZATIONS.
!              ---------------

!*    1.1  LOGICAL AND OTHER SCALAR INITIALIZATIONS

!     1.1.1  Define the "standard order" and pointers for post-processed GFL

! we compute here pointers YLPPIC%I[X] to locate GFL in LL_GFL and ZPPGFL,
! and the number of 2D fields (IPPGFL) we have to store in LL_GFL and ZPPGFL.
! Assumptions:
! - memory is conditionally allocated for multi-fields GFL and also for
!   LRCH4 and CH4S.
! - for the other GFL variables allocation is always done for the time being,
!   even for not post-processed fields.
! These pointers will be re-used for the non-derivative fields stored in ZGFLT0.

IPPGFL=0
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%IQ)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%IL)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%ILRAD)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%II)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%IIRAD)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%IA)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%IO3)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%IDAL)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%IDOM)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%IUAL)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%IUOM)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%IUEN)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%IUNEBH)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%IS)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%IRR)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%IG)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%IH)
CALL SUPTRPPGFL_POS(1,IPPGFL,YLPPIC%ITKE)
CALL SUPTRPPGFL_POS(MIN(NGHG,1),IPPGFL,YLPPIC%ILRCH4)
CALL SUPTRPPGFL_POS(MIN(NGHG,1),IPPGFL,YLPPIC%ICH4S)
CALL SUPTRPPGFL_POS(NGHG,IPPGFL,YLPPIC%IGHG)
CALL SUPTRPPGFL_POS(NTRAC,IPPGFL,YLPPIC%ITRAC)
CALL SUPTRPPGFL_POS(NAERO,IPPGFL,YLPPIC%IAERO)
CALL SUPTRPPGFL_POS(NGRG,IPPGFL,YLPPIC%IGRG)
CALL SUPTRPPGFL_POS(NGFL_EXT,IPPGFL,YLPPIC%IEXT)
CALL SUPTRPPGFL_POS(NNOGW,IPPGFL,YLPPIC%INOGW)
CALL SUPTRPPGFL_POS(NAEROUT,IPPGFL,YLPPIC%IAEROUT)
CALL SUPTRPPGFL_POS(NUVP,IPPGFL,YLPPIC%IUVP)
CALL SUPTRPPGFL_POS(NERA40,IPPGFL,YLPPIC%IERA40)

!     1.1.2  LL_GFL,LLAP_GFL,LLIN_GFL,IGFLCOD,IDIN_GFL

ALLOCATE(LL_GFL(IPPGFL))
ALLOCATE(LLAP_GFL(IPPGFL))
ALLOCATE(IGFLCOD(IPPGFL))
ALLOCATE(IDIN_GFL(IPPGFL))
ALLOCATE(LLIN_GFL(IPPGFL))

CALL POS_PREPGFL(IPPGFL,YDIN,YDGFL,YLPPIC,&
 & LL_GFL,LLAP_GFL,LLIN_GFL,IGFLCOD,IDIN_GFL)

!     1.1.3  LL_[X] for other variables, and ICUFCOD

! provisional piece of code, but later it would be desirable to replace
! TFP_CUF1 to TFP_CUF5 by TFP_CUF.
IF (NCUFNR>=1) ICUFCOD(1)=TFP_CUF1%ICOD
IF (NCUFNR>=2) ICUFCOD(2)=TFP_CUF2%ICOD
IF (NCUFNR>=3) ICUFCOD(3)=TFP_CUF3%ICOD
IF (NCUFNR>=4) ICUFCOD(4)=TFP_CUF4%ICOD
IF (NCUFNR>=5) ICUFCOD(5)=TFP_CUF5%ICOD

LL_PPUV = QFPTYPE%LL(TFP_U%ICOD) .OR. QFPTYPE%LL(TFP_V%ICOD) .OR. &
 & (QFPTYPE%LL(TFP_VOR%ICOD).AND.(QFPTYPE%ISKP(TFP_VOR%ICOD) == 2)) .OR. &
 & (QFPTYPE%LL(TFP_DIV%ICOD).AND.(QFPTYPE%ISKP(TFP_DIV%ICOD) == 2)) .OR. &
 & (QFPTYPE%LL(TFP_ABS%ICOD).AND.(QFPTYPE%ISKP(TFP_ABS%ICOD) == 2)) .OR. &
 & QFPTYPE%LL(TFP_KHI%ICOD) .OR. QFPTYPE%LL(TFP_PSI%ICOD)
LL_T    = QFPTYPE%LL(TFP_T%ICOD)
LL_P    = QFPTYPE%LL(TFP_P%ICOD)
LL_SP   = QFPTYPE%LL(TFP_SP%ICOD)
LL_LNSP = QFPTYPE%LL(TFP_LNSP%ICOD)
LL_PD   = QFPTYPE%LL(TFP_PD%ICOD)
LL_VW   = QFPTYPE%LL(TFP_VW%ICOD)
LL_WWS  = QFPTYPE%LL(TFP_WWS%ICOD)
LL_VD   = QFPTYPE%LL(TFP_VD%ICOD)
LL_Z    = QFPTYPE%LL(TFP_Z%ICOD)
LL_MG   = QFPTYPE%LL(TFP_MG%ICOD)
LL_HU   = QFPTYPE%LL(TFP_HU%ICOD)
LL_VV   = QFPTYPE%LL(TFP_VV%ICOD)
LL_ETAD = QFPTYPE%LL(TFP_ETAD%ICOD)
LL_MSL  = QFPTYPE%LL(TFP_MSL%ICOD)
LL_CUF(1:NCUFNR)=QFPTYPE%LL(ICUFCOD(1:NCUFNR))
LL_PV   = QFPTYPE%LL(TFP_PV%ICOD)
LL_TH   = QFPTYPE%LL(TFP_TH%ICOD)
LL_ETH  = QFPTYPE%LL(TFP_ETH%ICOD)
LL_VTG  = QFPTYPE%LL(TFP_VTG%ICOD)
LL_STD  = QFPTYPE%LL(TFP_STD%ICOD)
LL_SHD  = QFPTYPE%LL(TFP_SHD%ICOD)
LL_TWV  = QFPTYPE%LL(TFP_TWV%ICOD)
LL_FOL  = QFPTYPE%LL(TFP_FOL%ICOD)
LL_IET  = QFPTYPE%LL(TFP_IET%ICOD)
LL_SRE  = QFPTYPE%LL(TFP_SRE%ICOD)
LL_THV  = QFPTYPE%LL(TFP_THV%ICOD)
LL_THPW = QFPTYPE%LL(TFP_THPW%ICOD)
LL_WND  = QFPTYPE%LL(TFP_WND%ICOD)
LL_DIV  = QFPTYPE%LL(TFP_DIV%ICOD).AND.(QFPTYPE%ISKP(TFP_DIV%ICOD) == 1)
LL_VOR  = QFPTYPE%LL(TFP_VOR%ICOD).AND.(QFPTYPE%ISKP(TFP_VOR%ICOD) == 1) .OR. &
 & QFPTYPE%LL(TFP_ABS%ICOD).AND.(QFPTYPE%ISKP(TFP_ABS%ICOD) == 1)
LL_ABS  = QFPTYPE%LL(TFP_ABS%ICOD).AND.(QFPTYPE%ISKP(TFP_ABS%ICOD) == 1)
LL_PCAO = QFPTYPE%LL(TFP_PCAO%ICOD)
LL_TCAO = QFPTYPE%LL(TFP_TCAO%ICOD)
LL_UJET = QFPTYPE%LL(TFP_UJET%ICOD)
LL_VJET = QFPTYPE%LL(TFP_VJET%ICOD)
LL_PJET = QFPTYPE%LL(TFP_PJET%ICOD)
LL_PTB  = QFPTYPE%LL(TFP_PTB%ICOD)
LL_HTB  = QFPTYPE%LL(TFP_HTB%ICOD)
LL_HTPW = QFPTYPE%LL(TFP_HTPW%ICOD)
LL_MOCO = QFPTYPE%LL(TFP_MOCO%ICOD)
LL_RHO = QFPTYPE%LL(TFP_RHO%ICOD)
LL_QNH = QFPTYPE%LL(TFP_QNH%ICOD)
LL_FLASH= QFPTYPE%LL(TFP_FLASH%ICOD)
DO JVAR=1,JPOSVX2
  LL_FUA(JVAR) = QFPTYPE%LL(TFP_FUA(JVAR)%ICOD)
ENDDO
DO JVAR=1,JPOSFSU
  LL_FSU(JVAR) = QFPTYPE%LL(TFP_FSU(JVAR)%ICOD)
ENDDO

!     1.1.4  LLDYN, LLAP

! LLAP says if the call to APACHE must be done.
LLAP=(LFPART2 .AND. (CDCONF == 'H'.OR.CDCONF == 'S'.OR.CDCONF == 'F')).AND. &
 & (LL_PPUV.OR.LL_WND.OR.LL_T.OR.LL_THPW.OR.&
 & LL_PD.OR.LL_VD.OR.LL_VW.OR.LL_P.OR.ANY(LLAP_GFL))

! LLDYN says about the calculations of part 2.1 which must be done.
! * If CDCONF='B' or 'M' or 'V' or 'T' or 'S', LLDYN is set to T only
!   when a subset of the calculations under LLDYN (part 2.1) is required.
! * If CDCONF='H', LLDYN(3:4) is set to T only
!   when a subset of the calculations under LLDYN (part 2.1) is required;
!   LLDYN(1:2) is always set to T (calculation of geopotential height required).
! * In all the other cases: LLDYN=T.

! * LLDYN(1)=T => requires pressure departure calculation.
! * LLDYN(2)=T => requires geopotential height calculation.
! * LLDYN(3)=T => requires call to GPCTY
! * LLDYN(4)=T => requires calculation of "w" and -G*(Delta w)
! The following intermediate variables are defined:
! * LLAERO: call to POAERO is needed (uses geopot. as input)
! * LLPD: calculation of ZPDEP is needed
! * LLGEO: calculation of geopotential height is needed
! * LLCTY: call to GPCTY is needed
! * LLWW: calculation of "w" or -G*(Delta w) is needed
LLDYN(:)=.FALSE.
IF (CDCONF=='H'.OR.CDCONF == 'F') LLDYN(1:2)=.TRUE.
IF (LLAP) LLDYN(1)=.TRUE.
LLAERO=LL_UJET.OR.LL_VJET.OR.LL_PJET.OR.LL_PCAO.OR.LL_TCAO
LLPD=LL_PD.OR.LL_RHO
LLGEO=LL_Z.OR.LL_MG.OR.LLAERO.OR.LL_PTB.OR.LL_HTB.OR.&
 & LL_HTPW.OR.LL_MOCO
LLCTY=LL_VV.OR.LL_ETAD.OR.LL_VTG
LLWW=LL_VW.OR.LL_WWS.OR.LL_VD
IF(LLPD.OR.LLGEO.OR.LLCTY.OR.LLWW.OR.LL_FLASH) LLDYN(1)=.TRUE.
IF(LLGEO.OR.LLCTY.OR.LLWW.OR.LL_VTG) LLDYN(2)=.TRUE.
IF(LLCTY.OR.LLWW) LLDYN(3)=.TRUE.
IF(LLWW.OR.LL_FLASH) LLDYN(4)=.TRUE.

!     1.1.5  Other variables.

! LLPVO: call to GPPVO is required
LLPVO=(CDCONF == 'V' .OR. CDCONF == 'T').OR. &
 & (LL_PV.OR.LL_FOL.OR.LL_ETH.OR.LL_THV.OR.LL_TH)
! LLTDER: horizontal derivatives of T are required
LLTDER=LLDYN(1).OR.LLPVO
! LLUVDER: zonal derivatives of (U,V) are required
LLUVDER=LL_STD.OR.LL_SHD
! LLDIV0: horizontal divergence is required
LLDIV0=LLDYN(3).OR.LL_DIV.OR.LL_STD.OR.LL_SHD.OR.LL_MOCO
! LLVOR0: vorticity is required
LLVOR0=LLPVO.OR.LL_VOR.OR.LL_ABS.OR.LL_STD.OR.LL_SHD
! LLQDER: horizontal derivatives of specific humidity are required
LLQDER=LLDYN(1).OR.LL_MOCO

! --- Relaxation of thin layer hypothesis not yet coded ----------------------
LLVERCOR=.FALSE.
IIPROMVC=1
! ----------------------------------------------------------------------------

ILOLEV=1
IHILEV=KOPLEV
ISLCT1=1
ISLCT2=2

ZRHBNDS(1)=FPRHMIN
ZRHBNDS(2)=FPRHMAX

!     1.1.6  tests.

IF (LLUVDER .AND. .NOT.LUVDER) THEN
  CALL ABOR1 ('POS INTERNAL ERROR ON LUVDER')
ENDIF
IF (LLVOR0.AND. .NOT.LVOR) THEN
  CALL ABOR1 ('POS INTERNAL ERROR ON LVOR')
ENDIF
IF (LLQDER .AND. .NOT.(YDGFL%LLQL.AND.YDGFL%LLQM) .AND. .NOT.LSPRT) THEN
  CALL ABOR1 ('POS : NEEDS Q DERIVATIVES')
ENDIF
IF (LNH_GEOGW) CALL ABOR1('POS: lnh_geogw not yet coded')
IF (LFPART2.AND.LRWSDLR) CALL ABOR1('POS: case LFPART2.AND.LRWSDLR not yet coded')

!*    1.2  ARRAYS INITIALIZATIONS

!*      1.2.1   Constants

DO JROF=KST,KND
  ZGMAPP(JROF)=PGM(JROF)*PGM(JROF)
  ZOROGL(JROF)=POROGL(JROF)*PGM(JROF)
  ZOROGM(JROF)=POROGM(JROF)*PGM(JROF)
ENDDO

!*      1.2.2   ZGFLT0, ZGFLT0L, ZGFLT0M

ALLOCATE(ZGFLT0(KPROMA,0:KFLEVG,IPPGFL))
ZGFLT0(:,:,:)=0.0_JPRB
DO JVAR=1,IPPGFL
  IF (LLIN_GFL(JVAR)) THEN
    DO JLEV=1,KFLEVG
      DO JROF=KST,KND
        ZGFLT0(JROF,JLEV,JVAR)=PGFL(JROF,JLEV,IDIN_GFL(JVAR))
      ENDDO
    ENDDO
  ENDIF
ENDDO

DO JLEV=1,KFLEVG
  IF (YDGFL%LLQL) THEN
    ZQT0L(KST:KND,JLEV)=PGFL(KST:KND,JLEV,YDIN%IQL)*PGM(KST:KND)
  ENDIF
  IF (YDGFL%LLQM) THEN
    ZQT0M(KST:KND,JLEV)=PGFL(KST:KND,JLEV,YDIN%IQM)*PGM(KST:KND)
  ENDIF
  ZGRHLT0(KST:KND,JLEV)= &
   & ZGFLT0(KST:KND,JLEV,YLPPIC%IG)+ZGFLT0(KST:KND,JLEV,YLPPIC%IH)
ENDDO

!*      1.2.3   Z[X]T0 for GMV and GMVS

DO JLEV=1,KFLEVG
  ZUT0(KST:KND,JLEV)=PU(KST:KND,JLEV)*PGM(KST:KND)
  ZVT0(KST:KND,JLEV)=PV(KST:KND,JLEV)*PGM(KST:KND)
  IF (LLUVDER) THEN
    ZUT0L(KST:KND,JLEV)=PUL(KST:KND,JLEV)*ZGMAPP(KST:KND)
    ZVT0L(KST:KND,JLEV)=PVL(KST:KND,JLEV)*ZGMAPP(KST:KND)
  ENDIF
  IF (LLDIV0) THEN
    ZDIVT0(KST:KND,JLEV)=PDIV(KST:KND,JLEV)*ZGMAPP(KST:KND)
  ENDIF
  IF (LLVOR0) THEN
    ZVORT0(KST:KND,JLEV)=PVOR(KST:KND,JLEV)*ZGMAPP(KST:KND)
  ENDIF
  ZTT0(KST:KND,JLEV)=PT(KST:KND,JLEV)
  IF (LLTDER) THEN
    ZTT0L(KST:KND,JLEV)=PTL(KST:KND,JLEV)*PGM(KST:KND)
    ZTT0M(KST:KND,JLEV)=PTM(KST:KND,JLEV)*PGM(KST:KND)
  ENDIF
  IF (LNHDYN) THEN
    ZPDT0(KST:KND,JLEV)=PSPD(KST:KND,JLEV)
    ZPDT0L(KST:KND,JLEV)=PSPDL(KST:KND,JLEV)*PGM(KST:KND)
    ZPDT0M(KST:KND,JLEV)=PSPDM(KST:KND,JLEV)*PGM(KST:KND)
  ELSE
    ZPDT0(KST:KND,JLEV)=0.0_JPRB
    ZPDT0L(KST:KND,JLEV)=0.0_JPRB
    ZPDT0M(KST:KND,JLEV)=0.0_JPRB
  ENDIF
ENDDO
DO JROF=KST,KND
  ZPRESH(JROF,KFLEVG)=EXP(PSP(JROF))
  ZSP0L(JROF)=ZPRESH(JROF,KFLEVG)*PSPL(JROF)*PGM(JROF)
  ZSP0M(JROF)=ZPRESH(JROF,KFLEVG)*PSPM(JROF)*PGM(JROF)
ENDDO

!*      1.2.4 INITIALIZATION OF THE LOCAL PP FIELD ARRAYS ZPP[X]

ALLOCATE(ZPPGFL(KPROMA,KOPLEV,IPPGFL))

IF(YSD_XAD%NUMFLDS > 0) ALLOCATE(ZEXTRA(KPROMA,0:KFLEVG))
IF(YSD_XRD%NUMFLDS > 0) ALLOCATE(ZEXRAD(KPROMA,0:KFLEVG))

INFUA=COUNT(LL_FUA)
INFSU=COUNT(LL_FSU)
ALLOCATE(ZPPFUA(KPROMA,KOPLEV,INFUA))
ALLOCATE(ZPPFSU(KPROMA,INFSU))

! * GFL:
ZPPGFL(:,:,:)=0.0_JPRB

! * GMV and other non-GFL variables:
ZPPU(:,:)   =0.0_JPRB
ZPPV(:,:)   =0.0_JPRB
ZPPT(:,:)   =0.0_JPRB
ZPPUAP(:,:) =0.0_JPRB
ZPPSP(:)    =0.0_JPRB
ZPPLNSP(:)  =0.0_JPRB
ZPPPDEP(:,:)=0.0_JPRB
ZPPWVEL(:,:)=0.0_JPRB
ZPPMGDW(:,:)=0.0_JPRB
ZPPPHI(:,:) =0.0_JPRB
ZPPMG(:,:)  =0.0_JPRB
ZPPHRL(:,:) =0.0_JPRB
ZPPVVE(:,:) =0.0_JPRB
ZPPEVEL(:,:)=0.0_JPRB
ZPPMSLP(:)  =0.0_JPRB
ZPPCUF(:,:) =0.0_JPRB
ZPPPOTV(:,:)=0.0_JPRB
ZPPTETA(:,:)=0.0_JPRB
ZPPEPTH(:,:)=0.0_JPRB
ZPPVTG(:,:)=0.0_JPRB
ZPPSTDF(:,:)=0.0_JPRB
ZPPSHDF(:,:)=0.0_JPRB
ZPPTWV(:)   =0.0_JPRB
ZPPFOLDIN(:)=0.0_JPRB
ZPPIBET(:,:)=0.0_JPRB
ZPPSIMR(:,:)=0.0_JPRB
ZPPVIRT(:,:)=0.0_JPRB
ZPPTHPW(:,:)=0.0_JPRB
ZPPWIND(:,:)=0.0_JPRB
ZPPDIV(:,:) =0.0_JPRB
ZPPVOR(:,:) =0.0_JPRB
ZPPABS(:,:) =0.0_JPRB
ZPPPICAO(:) =0.0_JPRB
ZPPTICAO(:) =0.0_JPRB
ZPPUJET(:)  =0.0_JPRB
ZPPVJET(:)  =0.0_JPRB
ZPPPJET(:)  =0.0_JPRB
ZPPPTB(:,:)  =0.0_JPRB
ZPPHTB(:,:)  =0.0_JPRB
ZPPHTPW(:)  =0.0_JPRB
ZPPMOCO(:)  =0.0_JPRB
ZPPQNH(:)  =0.0_JPRB
ZPPRHO(:,:) =0.0_JPRB
ZPPR(:,:) =0.0_JPRB
ZPPFSU(:,:) =0.0_JPRB
ZPPFUA(:,:,:)=0.0_JPRB

!     ------------------------------------------------------------------

!*       2.    DYNAMICS.
!              ---------

!*      2.1.    CALCULATIONS ALSO DONE IN CPG_GP

! Calculations done in this part must remain consistent with CPG_GP ones.
! Additionally, the LFPART2 case is treated here (does not exist in CPG_GP).

!*      2.1.1   COMPUTATION OF Cp, R AND R/Cp

CALL GPRCP(KPROMA,KST,KND,KFLEVG,PQ=ZGFLT0(1,1,YLPPIC%IQ),&
 & PQI=ZGFLT0(1,1,YLPPIC%II),PQL=ZGFLT0(1,1,YLPPIC%IL),&
 & PQR=ZGFLT0(1,1,YLPPIC%IRR),PQS=ZGFLT0(1,1,YLPPIC%IS),PQG=ZGRHLT0(1,1),&
 & PCP=ZCP,PR=ZR,PKAP=ZKAP)

IF (LL_RHO) THEN
  CALL GPRCP(KPROMA,KST,KND,KFLEVG,PQ=ZGFLT0(1,1,YLPPIC%IQ),&
   & PQI=ZGFLT0(1,1,YLPPIC%II),PQL=ZGFLT0(1,1,YLPPIC%IL),PR=ZR2)
ENDIF

!*      2.1.2   MODEL LEVEL PRESSURES

! ZPRESH, ZXYB and ZPRESF.
CALL GPHPRE(KPROMA,KFLEVG,KST,KND,YRVAB,ZPRESH,PXYB=ZXYB,PRESF=ZPRESF)

IF (LLDYN(1)) THEN
  ! 1/surface pressure and output of GPGRXYB.
  IF(LVERTFE) THEN
    DO JLEV=1,KFLEVG
      DO JROF=KST,KND
        ZRPRESF(JROF,JLEV)=1.0_JPRB/ZPRESF(JROF,JLEV)
      ENDDO
    ENDDO
  ENDIF
  CALL GPGRXYB(KPROMA,KST,KND,KFLEVG,.FALSE.,YRVAB,ZSP0L,ZSP0M,ZXYB,ZXYBDER)
ENDIF

!*      2.1.3   COMPUTATION OF RT and grad(RT)

IF (LLDYN(1)) THEN
  CALL GPRT(LSPRT,KPROMA,KST,KND,KFLEVG,RD,RV,&
   & ZR,ZTT0(1,1),ZTT0L,ZTT0M,ZQT0L,ZQT0M,ZRT,ZRTL,ZRTM)
ENDIF

!*      2.1.4   NH PRESSURE DEPARTURE

IF (LLDYN(1)) THEN
  IF (LNHDYN) THEN
    IF (LFPART2) THEN
      ! ZPDT0 contains pre-prehyd
      ! Gradients of pressure departure quantities are not required in this case
      DO JLEV=1,KFLEVG
        DO JROF=KST,KND
          ZPDEP(JROF,JLEV)=ZPDT0(JROF,JLEV)
          ZNHPREF(JROF,JLEV)=ZPDT0(JROF,JLEV)+ZPRESF(JROF,JLEV)
          ZNHPPI(JROF,JLEV)=ZNHPREF(JROF,JLEV)/ZPRESF(JROF,JLEV)
          ZRNHPPI(JROF,JLEV)=ZPRESF(JROF,JLEV)/ZNHPREF(JROF,JLEV)
          ZRRED(JROF,JLEV)=ZR(JROF,JLEV)*ZPRESF(JROF,JLEV)/ZNHPREF(JROF,JLEV)
        ENDDO
      ENDDO
    ELSE
      ! ZPDT0 contains pressure departure prognostic variable.
      CALL GNHPRE(KPROMA,KFLEVG,KST,KND,ZPDT0(1,1),ZPRESF, &
       & ZNHPREF,ZNHPPI,ZRNHPPI,ZPDEP(1,1))
      DO JLEV=1,KFLEVG
        DO JROF=KST,KND
          ZRRED(JROF,JLEV)=ZR(JROF,JLEV)/ZNHPPI(JROF,JLEV)
        ENDDO
      ENDDO
      CALL GNHGRPRE(KPROMA,KFLEVG,KST,KND,ZXYB(1,1,YYTXYBPP%M_RTGR),ZSP0L,ZSP0M, &
       & ZNHPREF,ZPDT0L,ZPDT0M, &
       & ZUSL_NHPREL,ZUSL_NHPREM,ZUSL_LNNHPREFL,ZUSL_LNNHPREFM,ZQCHAL,ZQCHAM)
    ENDIF
  ELSE
    ZPDEP(KST:KND,1:KFLEVG)=0.0_JPRB
    ZNHPPI(KST:KND,1:KFLEVG)=1.0_JPRB
    ZRNHPPI(KST:KND,1:KFLEVG)=1.0_JPRB
    ZNHPREF(KST:KND,1:KFLEVG)=ZPRESF(KST:KND,1:KFLEVG)
    ZQCHAL(KST:KND,1:KFLEVG)=0.0_JPRB
    ZQCHAM(KST:KND,1:KFLEVG)=0.0_JPRB
    ZRRED(KST:KND,1:KFLEVG)=ZR(KST:KND,1:KFLEVG)
  ENDIF
ENDIF

!*      2.1.5   MODEL LEVEL GEOPOTENTIALS AND THEIR GRADIENTS
!               ALSO COMPUTES THE RADIUS "r" AND ITS GRADIENT IF LRWSDLR=T

IF (LLDYN(2)) THEN
  ! In this case we must compute these quantities before the call to GPCTY
  ! because we use PRDLR0 in GPCTY, and output of GNHDLR is used to compute
  ! the geopotential height with surface orography=POROG.
  IF (LNHDYN.AND.LRWSDLR) THEN
    ! "r/a", "a/r" and geopotential height.
    CALL GNHDLR(KPROMA,KFLEVG,KST,KND,ZTT0(1,1),ZR,ZRNHPPI, &
     & ZXYB(1,1,YYTXYBPP%M_LNPR),ZXYB(1,1,YYTXYBPP%M_ALPH),POROG,ZRDLR,PHIH=ZGEOPH,PHIF=ZGEOPF(1,1))
    ! "grad(r/a)" and gradient of geopotential height.
    CALL GNHGRDLR(KPROMA,KFLEVG,KST,KND,&
     & ZTT0(1,1),ZR,ZRNHPPI,ZRTL,ZRTM,&
     & ZXYB(1,1,YYTXYBPP%M_LNPR),ZXYB(1,1,YYTXYBPP%M_ALPH),ZXYBDER,&
     & ZQCHAL,ZQCHAM,ZOROGL,ZOROGM,ZRDLR,&
     & ZGPFL,ZGPFM,ZGPHL,ZGPHM)
  ELSE
    ! Geopotential height with surface orography=POROG
    DO JROF=KST,KND
      ZGEOPH(JROF,KFLEVG)=POROG(JROF)
    ENDDO
    CALL GPGEO(KPROMA,KST,KND,KFLEVG,ZGEOPH,ZGEOPF(1,1),&
     & ZTT0(1,1),ZRRED,ZXYB(1,1,YYTXYBPP%M_LNPR),ZXYB(1,1,YYTXYBPP%M_ALPH))
    IF (.NOT.LFPART2) THEN
      CALL GPGRGEO(LNHDYN,KPROMA,KST,KND,KFLEVG,&
       & ZRNHPPI,ZTT0(1,1),ZR,ZRTL,ZRTM,&
       & ZXYB(1,1,YYTXYBPP%M_LNPR),ZXYB(1,1,YYTXYBPP%M_ALPH),ZXYBDER,&
       & ZQCHAL,ZQCHAM,ZOROGL,ZOROGM,ZGPFL,ZGPFM,ZGPHL,ZGPHM)
    ENDIF
  ENDIF
ENDIF

!*      2.1.6   COMPUTE QUANTITIES NECESSARY FOR OPTION LVERCOR=.T..

IF (LLDYN(3)) THEN
  IF (LVERCOR) THEN
    CALL GPVCRS(KPROMVC,KST,KND,KFLEVG,ZPRESF,ZPRESH,ZVCRS,&
     & PRTGR=ZXYB(1,1,YYTXYBPP%M_RTGR),PREHYDSL=ZSP0L,PREHYDSM=ZSP0M)
  ENDIF
  IF (.NOT.LVERCOR) THEN
    ZVCRS(1:KPROMVC,1:KFLEVG,YYTVCPP%M_VCRSSAF)=1.0_JPRB
    ZVCRS(1:KPROMVC,0:KFLEVG,YYTVCPP%M_VCRSSAH)=1.0_JPRB
    ZVCRS(1:KPROMVC,1:KFLEVG,YYTVCPP%M_VCASRSF)=1.0_JPRB
    ZVCRS(1:KPROMVC,0:KFLEVG,YYTVCPP%M_VCASRSH)=1.0_JPRB
    ZVCRS(1:KPROMVC,1:KFLEVG,YYTVCPP%M_VCRSSAFL)=0.0_JPRB
    ZVCRS(1:KPROMVC,1:KFLEVG,YYTVCPP%M_VCRSSAFM)=0.0_JPRB
  ENDIF
ENDIF

!*      2.1.7   INTEGRATION OF HYDROSTATIC AND CALCULATION OF etadot,
!               etadot Dprehyd/Dt and omega/prehyd.

IF (LLDYN(3)) THEN
  CALL GPCTY(KPROMA,KPROMVC,KST,KND,KFLEVG,LVERCOR,.FALSE.,YRVAB,YRVETA,&
   & ZUT0(1,1),ZVT0(1,1),ZDIVT0(1,1),PEV,&
   & ZXYB,ZSP0L,ZSP0M,ZVCRS,ZRPRESF,ZCTY,LDRWSDLR=LRWSDLR,PRDLR=ZRDLR)
  ! etadot is always computed at full levels.
  DO JLEV=1,KFLEVG
    IF(LVERTFE) THEN
      ZETADOT(KST:KND,JLEV)=ZCTY(KST:KND,JLEV,YYTCTYPP%M_EVEL) &
       & *(YRVETA%VETAH(JLEV)-YRVETA%VETAH(JLEV-1))*ZXYB(KST:KND,JLEV,YYTXYBPP%M_RDELP)
    ELSE
      ZETADOT(KST:KND,JLEV)=&
       & 0.5_JPRB*(ZCTY(KST:KND,JLEV,YYTCTYPP%M_EVEL)+ZCTY(KST:KND,JLEV-1,YYTCTYPP%M_EVEL)) &
       & *(YRVETA%VETAH(JLEV)-YRVETA%VETAH(JLEV-1))*ZXYB(KST:KND,JLEV,YYTXYBPP%M_RDELP)
    ENDIF
  ENDDO
  !nyc IF ( LSFORC .AND. LSW_FRC ) : call to GPCTY_FORC not yet coded here.
ENDIF

!*      2.1.8   COMPUTES HALF-LEVEL AND SURFACE WINDS, "NHX", "w" AND -G*Dw.

IF (LLDYN(4)) THEN

  ! Half-level and surface winds:
  CALL GPHLWI(KPROMA,KST,KND,ZXYB(1,1,YYTXYBPP%M_LNPR),ZXYB(1,1,YYTXYBPP%M_ALPH),ZUVH(1,1,YYTHWPP%M_WWI))
  CALL GPHLUV(KPROMA,KST,KND,ZUT0(1,1),ZVT0(1,1),ZUVH)
  CALL GPUVS(KFLEVG,KPROMA,KST,KND,.FALSE.,ZUT0(1,1),ZVT0(1,1),ZUS,ZVS)

  ! Term NHX:
  IF (.NOT.LFPART2) THEN
    IF (LSLAG.AND.LNHDYN.AND.(NVDVAR==4).AND.(ND4SYS==2)) THEN
      ZNHX(KST:KND,1:KFLEVG)=PNHX(KST:KND,1:KFLEVG)
    ELSE
      CALL GPXX(KFLEVG,KPROMA,KST,KND,ZGPHL,ZGPHM,ZGPFL,ZGPFM,ZXYB(1,1,YYTXYBPP%M_LNPR), &
       & ZNHPPI,ZTT0(1,1),ZR,ZUT0(1,1),ZVT0(1,1),ZUVH(1,0,YYTHWPP%M_UH),ZUVH(1,0,YYTHWPP%M_VH), &
       & ZNHX,LDRWSDLR=LRWSDLR,PRDLR=ZRDLR)
    ENDIF
  ENDIF

  ! Vertical divergence:
  IF (.NOT.LFPART2) THEN
    IF (LNHDYN) THEN
      IF (NVDVAR == 3) THEN
        ! PSVD contains 'dver'
        ZDVER(KST:KND,1:KFLEVG)=PSVD(KST:KND,1:KFLEVG)
      ELSEIF (NVDVAR == 4) THEN
        ! PSVD contains 'dver + NHX'
        DO JLEV=1,KFLEVG
          DO JROF=KST,KND
            ZDVER(JROF,JLEV)=PSVD(JROF,JLEV)-ZNHX(JROF,JLEV)
          ENDDO
        ENDDO
      ENDIF
    ELSE
      ! dver_hyd=-(R/Rd)*(D+X+(cv/cp)(omega/prehyd))
      DO JLEV=1,KFLEVG
        DO JROF=KST,KND
          ZDVER(JROF,JLEV)=(-ZR(JROF,JLEV)/RD)*(ZDIVT0(JROF,JLEV) &
           & +ZNHX(JROF,JLEV)+(1._JPRB-ZKAP(JROF,JLEV))*ZCTY(JROF,JLEV,YYTCTYPP%M_VVEL))
        ENDDO
      ENDDO
    ENDIF
  ENDIF

  ! Vertical velocity "w", and -G*(Delta w):
  Z1SRG=1.0_JPRB/RG
  ! ky: for the time being, LLVFE is set to .F. anytime because half level
  !     values of ZWW may be required in parts 4 and 5, and because
  !     hard coded FD code for conversions [Delta(w)] <-> w is also present
  !     in routines like SPECFITA and SPECFITG; the VFE case
  !     must be properly studied and coded later. Provisionally, a FD
  !     calculation of "w" at half levels is kept, even in a FULL-POS task
  !     running with LVERTFE.AND.LVFE_GW
  !! LLVFE=LVERTFE.AND.LVFE_GW
  LLVFE=.FALSE.
  IF (LFPART2) THEN
    ! This piece of code is never called if LNHDYN=F.
    ! Input PSVD contains -G*Delta w.
    ZMGDW(KST:KND,1:KFLEVG)=PSVD(KST:KND,1:KFLEVG)
    LLGWF=.FALSE.
    LLGDWI=.TRUE.
    CALL GPGW(KFLEVG,KPROMA,KST,KND,LLGWF,LLGDWI, &
     & ZOROGL,ZOROGM,ZXYB(1,1,YYTXYBPP%M_LNPR),ZXYB(1,1,YYTXYBPP%M_ALPH),ZUS,ZVS,ZTT0(1,1),ZMGDW,ZRNHPPI,ZGW,ZGWF, &
     & LDVFE=LLVFE, &
     & LDRWSDLR=LRWSDLR,LDRWSDLR2=LRWSDLR2,PRDLR=ZRDLR)
    IF (LLVFE) THEN
      ZWW(KST:KND,1:KFLEVG)=ZGWF(KST:KND,1:KFLEVG)*Z1SRG
    ELSE
      ZWW(KST:KND,0:KFLEVG)=ZGW(KST:KND,0:KFLEVG)*Z1SRG
    ENDIF
  ELSE
    ! Input ZDVER contains "dver"
    LLGWF=.FALSE.
    LLGDWI=.FALSE.
    CALL GPGW(KFLEVG,KPROMA,KST,KND,LLGWF,LLGDWI, &
     & ZOROGL,ZOROGM,ZXYB(1,1,YYTXYBPP%M_LNPR),ZXYB(1,1,YYTXYBPP%M_ALPH),ZUS,ZVS,ZTT0(1,1),ZDVER,ZRNHPPI,ZGW,ZGWF, &
     & LDVFE=LLVFE, &
     & LDRWSDLR=LRWSDLR,LDRWSDLR2=LRWSDLR2,PRDLR=ZRDLR,PGDW=ZGDW)
    IF (LLVFE) THEN
      ZWW(KST:KND,1:KFLEVG)=ZGWF(KST:KND,1:KFLEVG)*Z1SRG
      ZMGDW(KST:KND,1:KFLEVG)=-ZGDW(KST:KND,1:KFLEVG)
    ELSE
      ZWW(KST:KND,0:KFLEVG)=ZGW(KST:KND,0:KFLEVG)*Z1SRG
      DO JLEV=1,KFLEVG
        DO JROF=KST,KND
          ZMGDW(JROF,JLEV)=ZGW(JROF,JLEV-1)-ZGW(JROF,JLEV)
        ENDDO
      ENDDO
    ENDIF
  ENDIF

ENDIF

!*     2.1.8.5 PUT GEOMETRIC VERTICAL VELOCITY (m/s) IN OUTPUT ARRAY
!            w is needed for lightning diagnostic parametrization
!              if NOT VFE, use value from upper half level
  
DO JLEV=1,KFLEVG
  DO JROF=KST,KND
    IF (.NOT.LLVFE) THEN
      PWW(JROF,JLEV)=ZWW(JROF,JLEV-1)
    ELSE
      PWW(JROF,JLEV)=ZWW(JROF,JLEV)
    ENDIF
  ENDDO
ENDDO
  


!*      2.1.9   CONVERT MODEL LEVEL PRESSURES FOR LRWSDLR=T

! Provisional treatment of the deep-layer NH equations.
! * parts 2.2 and 2.3 ignore deep-layer effects.
! * interpolations only use true pressures, and not the apparent variables
!   used in the dynamics.
! * retrieval is done for prehyd, pre, and output data of GPHPRE.
! * transformation is ignored for output data of GPGRXYB and GPCTY (I don't know
!   how to do that), and we assume for example that Dpresim/Dt is very close
!   to Dprehyd/Dt.
IF (LNHDYN.AND.LRWSDLR) THEN
  ! Save presim.
  ZPRESIMF(KST:KND,1:KFLEVG)=ZPRESF(KST:KND,1:KFLEVG)
  ! Transform "presim" quantities into "prehyd" quantities.
  CALL GNHDLRB(KPROMA,KST,KND,.TRUE.,POROG,ZTT0(1,1),ZR,ZPDT0(1,1),ZPRESF,ZPRESH, &
   & ZXYB(1,1,YYTXYBPP%M_DELP),ZXYB(1,1,YYTXYBPP%M_RDELP),ZXYB(1,1,YYTXYBPP%M_LNPR),ZXYB(1,1,YYTXYBPP%M_ALPH),&
   & PRPREHO=ZXYB(1,1,YYTXYBPP%M_RPRE),PRPPO=ZXYB(1,1,YYTXYBPP%M_RPP))
  ! Convert ZRRED from (pre/presim)*R to (pre/prehyd)*R.
  IF (LLDYN(1)) THEN
    DO JLEV=1,KFLEVG
      DO JROF=KST,KND
        ZRRED(JROF,JLEV)=ZRRED(JROF,JLEV)*ZPRESIMF(JROF,JLEV)/ZPRESF(JROF,JLEV)
        ZPDEP(JROF,JLEV)=ZPDEP(JROF,JLEV)+ZPRESIMF(JROF,JLEV)-ZPRESF(JROF,JLEV)
      ENDDO
    ENDDO
  ENDIF
  ! From now on PSP should not be used any more in POS. Use ZPRESH instead.
ENDIF

!*      2.2.    ADDITIONAL DYNAMICAL DIAGNOSTICS

! We find there all dynamical quantities which are not required in CPG_GP,
! like potential vorticity, potential temperature, reflectivities.
! Deep-layer effects are ignored in these calculations.

!*      2.2.1   ADDITIONAL CALCULATIONS OF GEOPOTENTIAL, USING
!               ALTERNATE OROGRAPHY.

IF (LLAP) THEN
  ! Geopotential height with surface orography=PXORO
  IF (LNHDYN.AND.LRWSDLR) THEN
    CALL ABOR1(' POS 2.2.1: case not yet coded')
  ELSE
    ZXGEOPH(KST:KND,KFLEVG)=PXORO(KST:KND)
    CALL GPGEO(KPROMA,KST,KND,KFLEVG,ZXGEOPH,ZUSL_GEOPF,&
     & ZTT0(1,1),ZRRED,ZXYB(1,1,YYTXYBPP%M_LNPR),ZXYB(1,1,YYTXYBPP%M_ALPH))
  ENDIF
ENDIF

!*      2.2.2   RELATIVE HUMIDITY

IF (LL_HU.OR.LLAP) THEN
  CALL GPRH(.FALSE.,KPROMA,KST,KND,KFLEVG,ZRHBNDS(2),ZRHBNDS(1),&
   & ZGFLT0(1,1,YLPPIC%IQ),ZTT0(1,1),ZPRESF,ZES,ZRHF(1,1))
ENDIF

!*      2.2.3   MODEL LEVEL POTENTIAL VORTICITY AND POTENTIAL TEMPERATURE

IF (LLPVO) THEN
  ! ky: valid for thin layer models.
  CALL GPPVO(KPROMA,IIPROMVC,KST,KND,KFLEVG,&
   & ZPRESF,ZXYB(1,1,YYTXYBPP%M_RDELP),ZKAP,PRCORI,&
   & ZVORT0(1,1),ZUT0(1,1),ZVT0(1,1),ZTT0(1,1),&
   & ZTT0M,ZTT0L,ZSP0M,ZSP0L,ZTOUP(1,1),ZTETA(1,1))  
ENDIF

!*      2.2.4   MODEL LEVEL DEFORMATIONS

IF ( LLUVDER ) THEN
  DO JLEV=1,KFLEVG
    DO JROF=KST,KND
      ZSTDF(JROF,JLEV)=ZUT0L(JROF,JLEV)-0.5_JPRB*ZDIVT0(JROF,JLEV)
      ZSHDF(JROF,JLEV)=ZVT0L(JROF,JLEV)-0.5_JPRB*ZVORT0(JROF,JLEV)
    ENDDO
  ENDDO
ENDIF

!*      2.2.5   MODEL LEVEL EQUIVALENT POTENTIAL TEPERATURE

IF (LL_ETH) THEN
  CALL GPEPT(KPROMA,KST,KND,KFLEVG,ZTETA(1,1),ZTT0(1,1),&
   & ZPRESF,ZEPTH(1,1))  
ENDIF

!*      2.2.6   MODEL LEVEL ISOBARIC EQUIVALENT TEMPERATURE

IF (LL_IET) THEN
  CALL GPIET(KPROMA,KST,KND,KFLEVG,ZGFLT0(1,1,YLPPIC%IQ),ZTT0(1,1),ZCP,&
   & ZPRESF,ZIBET(1,1))  
ENDIF

!*      2.2.7    MODEL LEVEL SIMULATED REFLECTIVITIES

IF (LL_SRE) THEN
  LLUNIT=.TRUE.
  CALL GPPRS0D(KPROMA,KST,KND,KFLEVG,ZTT0(1,1),ZGFLT0(1,1,YLPPIC%IQ),&
   & ZGFLT0(1,1,YLPPIC%IL),ZGFLT0(1,1,YLPPIC%IRR),ZGFLT0(1,1,YLPPIC%II),&
   & ZGFLT0(1,1,YLPPIC%IS),ZGRHLT0(1,1),LLUNIT,ZSIMR(1,1))
ENDIF

!*      2.2.8    MODEL LEVEL thetav

IF (LL_THV) THEN
  CALL PPCVIRT(KPROMA,KST,KND,KFLEVG,ZTETA(1,1),ZGFLT0(1,1,YLPPIC%IQ),&
   & PQL=ZGFLT0(1,1,YLPPIC%IL),PQR=ZGFLT0(1,1,YLPPIC%IRR),&
   & PQS=ZGFLT0(1,1,YLPPIC%IS),&
   & PQG=ZGRHLT0(1,1),PQI=ZGFLT0(1,1,YLPPIC%II),PTV=ZVIRT(1,1))
ENDIF

!*      2.2.9   VERTICAL GRADIENT OF TEMPERATURE

IF (LL_VTG) THEN
  ZVTGH(KST:KND,1)=0
  ZVTGH(KST:KND,0)=0
  DO JLEV=2,KFLEVG
    ZVTGH(KST:KND,JLEV)=(ZTT0(KST:KND,JLEV-1)-ZTT0(KST:KND,JLEV))/ &
       &                (ZGEOPH(KST:KND,JLEV-1)-ZGEOPH(KST:KND,JLEV))*RG
  ENDDO
  DO JLEV=1,KFLEVG
    ZVTGF(KST:KND,JLEV)=0.5_JPRB*(ZVTGH(KST:KND,JLEV)+ZVTGH(KST:KND,JLEV-1))
  ENDDO
ENDIF







! From now on no additional GP.. or GNH.. routine must be called in this routine.
! (excepted hidden GPGEO in PPGEOP and some hidden GP.. routines in APACHE,
! for system ESCALE).

!     ------------------------------------------------------------------
!*       3.    PREPARE FOR INTERPOLATIONS.
!              ---------------------------

LL_TSTAR=LL_MOCO.OR.LL_MG.OR.LL_T.OR.LL_THPW.OR.LL_Z.OR.LL_MSL.OR.&
        &LL_QNH.OR.LL_RHO.OR.(CDCONF == 'H').OR.(CDCONF == 'F')

!*      3.1      COMPUTE SURFACE TEMPERATURE AND LAPSE RATE.

IF (LL_MOCO.OR.LL_MG.OR.LL_T.OR.LL_THPW.OR.LL_Z.OR.LL_MSL.OR.CDCONF == 'H'.OR.&
   & CDCONF == 'F') THEN  
  CALL CTSTAR(KPROMA,KST,KND,ZTT0(1,NLEXTRAP),&
   & ZPRESH(1,KFLEVG),ZPRESF(1,NLEXTRAP),POROG,ZTSTAR,ZT0)  
ENDIF

!*      3.2      COMPUTE POST-PROCESSING LEVELS

IF (CDCONF == 'B') THEN
  DO JLEVP=1,KOPLEV
    DO JROF=KST,KND
      ZPRES(JROF,JLEVP)=PXLEV(JLEVP)
      ZLNPRES(JROF,JLEVP)=LOG(PXLEV(JLEVP))
    ENDDO
  ENDDO
ELSEIF (CDCONF == 'H') THEN
  DO JLEVP=1,KOPLEV
    DO JROF=KST,KND
      ZGEOP(JROF,JLEVP)=RG*PXLEV(JLEVP)+POROG(JROF)
    ENDDO
  ENDDO
  CALL FPPS(KPROMA,KST,KND,KFLEVG,KOPLEV,ZGEOPH,ZGEOP,ZTT0(1,1),&
   & ZR,ZPRESH,PTSI,ZPRES)  
  DO JLEVP=1,KOPLEV
    DO JROF=KST,KND
      ZLNPRES(JROF,JLEVP)=LOG(ZPRES(JROF,JLEVP))
    ENDDO
  ENDDO
ELSEIF (CDCONF == 'V') THEN
  DO JLEVP = 1 , KOPLEV
     CALL PPLTP (KPROMA,KST,KND,KFLEVG,ZPRESF,&
      & ZTOUP(1,1),PXLEV(JLEVP),.FALSE.,PRCORI,&
      & ZPRES(1,JLEVP),ZLNPRES(1,JLEVP),&
      & ZWORK)
  ENDDO
ELSEIF (CDCONF == 'T') THEN
  DO JLEVP=1,KOPLEV
    CALL PPLTETA(KPROMA,KST,KND,KFLEVG,ZPRESF,ZTETA(1,1),&
     & PXLEV(JLEVP),ZPRES(1,JLEVP),ZLNPRES(1,JLEVP))  
  ENDDO
ELSEIF (CDCONF == 'K') THEN
  DO JLEVP=1,KOPLEV
    ZXTEMP=ABS(PXLEV(JLEVP))
    LLTOP= (PXLEV(JLEVP) < 0.0_JPRB)
    CALL PPLTEMP(KPROMA,KST,KND,KFLEVG,ZGEOPF(1,1),ZTT0(1,1),&
     & ZXTEMP,LLTOP,ZGEOP(1,JLEVP))
  ENDDO
  CALL FPPS(KPROMA,KST,KND,KFLEVG,KOPLEV,ZGEOPH,ZGEOP,ZTT0(1,1), &
   & ZR,ZPRESH,PTSI,ZPRES)
  DO JLEVP=1,KOPLEV
    DO JROF=KST,KND
      ZLNPRES(JROF,JLEVP)=LOG(ZPRES(JROF,JLEVP))
    ENDDO
  ENDDO
ELSEIF (CDCONF == 'F') THEN
  DO JLEVP=1,KOPLEV
    DO JROF=KST,KND
      ZGEOP(JROF,JLEVP)=RG*PXLEV(JLEVP)
    ENDDO
  ENDDO
  CALL FPPS(KPROMA,KST,KND,KFLEVG,KOPLEV,ZGEOPH,ZGEOP,ZTT0(1,1),&
   & ZR,ZPRESH,PTSI,ZPRES)
  DO JLEVP=1,KOPLEV
    DO JROF=KST,KND
      ZLNPRES(JROF,JLEVP)=LOG(ZPRES(JROF,JLEVP))
    ENDDO
  ENDDO
ELSEIF (CDCONF == 'S') THEN
  IF (LFPART2) THEN
    IF (KFPLEV /= KOPLEV) THEN
      CALL ABOR1('ETA LEVELS SELECTION IMPOSSIBLE')
    ENDIF
  ELSE
    IF (LFPOSVER) THEN
      ! cf. PPLETA but without (useless) call to GPHPRE.
      DO JLEVP=1,KOPLEV
        IPTRLEV(JLEVP)=INT(PXLEV(JLEVP),JPIM)
        ZPRES(KST:KND,JLEVP)=ZPRESF(KST:KND,IPTRLEV(JLEVP))
        ZPH(KST:KND,JLEVP,1)=ZPRESH(KST:KND,IPTRLEV(JLEVP))
        ZPH(KST:KND,JLEVP,0)=ZPRESH(KST:KND,IPTRLEV(JLEVP)-1)
      ENDDO
    ELSE
      IF (LRWSDLR) THEN
        ! ky: if LRWSDLR, pressure calculations inside PPLETA are currently
        !     not correct, and not easy to adapt.
        !     GPHPRE is only valid for "presim".
        !     But the output data ZPRES and ZPH should be "prehyd" pressures. 
        !     Not easy to manage.
        CALL ABOR1(' POS: PPLETA not properly updated for LRWSDLR')
      ENDIF
      CALL PPLETA(KPROMA,KST,KND,KFPLEV,KOPLEV,PXLEV,&
       & ZPRESH(1,KFLEVG),ZPRES,ZPH(1,1,1),ZPH(1,1,0),PVAH=PVAH,PVBH=PVBH)  
    ENDIF
    DO JLEVP=1,KOPLEV
      DO JROF=KST,KND
        ZLNPRES(JROF,JLEVP)=LOG(ZPRES(JROF,JLEVP))
      ENDDO
    ENDDO
  ENDIF
ENDIF

!*      3.3     SET UP HELP-ARRAYS FOR VERTICAL INTERPOLATION

IF (CDCONF /= 'M') THEN
  CALL PPINIT(KPROMA,KST,KND,KFLEVG,KPPM,ZPRESH,ZPRESF,ZPXP,ZPXPD)
ENDIF

!*      3.4     FIND MODEL LEVEL UNDER SPECIFIED OUTPUT LEVELS

IF ( CDCONF /= 'M' .AND.&
   & ((CDCONF /= 'H'.AND.CDCONF /= 'S'.AND.CDCONF /= 'F').OR..NOT.LFPART2) ) THEN  
  CALL PPFLEV(KPROMA,KST,KND,KFLEVG,KOPLEV,KPPM,ZPRES,ZPRESH,&
   & ZPRESF,ILEVB,LLBELO,LLBELS,LLBLOW,LLBLES)  
  IF (LL_VD.AND.CDCONF == 'S') THEN
    DO JH=0,1
      CALL PPFLEV(KPROMA,KST,KND,KFLEVG,KOPLEV,KPPM,&
       & ZPH(1,1,JH),ZPRESH,ZPRESF,&
       & ILEVBH(1,1,1,JH),LLBELOH(1,1,JH),&
       & LLBELSH(1,1,JH),LLBLOWH(1,JH),LLBLESH(1,JH))  
    ENDDO
  ENDIF
ENDIF

!*      3.5     COMPUTE ICAO ATMOSPHERE

IF (CDCONF /= 'M'.AND.CDCONF /= 'V'.AND.(LL_T.OR.LL_THPW.OR.LL_Z.OR.LL_MG.OR.LL_RHO)) THEN
  CALL PPSTA('PPREF',KPROMA,KST,KND,KFLEVG+1,1,ZPXP(1,0,2),ZPXP(1,0,4),ZSTTF,&
   & ZSTZF)
ENDIF

!     ------------------------------------------------------------------
!*       4.    POST-PROCESSING (INTERPOLATIONS).
!              ---------------------------------

!*    4.1   TRADITIONAL GFL FIELDS

IF ((CDCONF /= 'H'.AND.CDCONF /= 'S'.AND.CDCONF /= 'F').OR..NOT.LFPART2) THEN

  DO JVAR=1,IPPGFL
    IF (LL_GFL(JVAR)) THEN
      IF (CDCONF /= 'M') THEN
        CALL PPQ(KPROMA,KST,KND,KFLEVG,KOPLEV,ILOLEV,KPPM,&
         & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZGFLT0(1,0,JVAR),ZPPGFL(1,1,JVAR))
      ELSE
        DO JLEVP=1,KOPLEV
          ILEVM=INT(PXLEV(JLEVP))
          ZPPGFL(KST:KND,JLEVP,JVAR)=ZGFLT0(KST:KND,ILEVM,JVAR)
        ENDDO
      ENDIF
    ENDIF
  ENDDO

ENDIF

!*    4.2   TRADITIONAL GMV+GMVS FIELDS

IF ((CDCONF /= 'H'.AND.CDCONF /= 'S'.AND.CDCONF /= 'F').OR..NOT.LFPART2) THEN

!*      4.2.1   WINDS :

  IF (LL_PPUV.OR.LL_WND) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPUV(KPROMA,KST,KND,KFLEVG,KOPLEV,ILOLEV,KPPM,ILEVB,LLBELO,LLBLOW,&
       & ZLNPRES,ZPXP,ZPXPD,ZUT0,ZVT0,ZPPU,ZPPV)  
    ELSE
      DO JLEVP=1,KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPU(KST:KND,JLEVP)=ZUT0(KST:KND,ILEVM)
        ZPPV(KST:KND,JLEVP)=ZVT0(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

!*      4.2.2    TEMPERATURE :

  IF (LL_T .OR. LL_THPW .OR. LL_RHO .AND. CDCONF /= 'K' ) THEN
    IF (CDCONF == 'V') THEN
      ! Do not use PPT because it contains an implicit definition of
      ! the tropopause
      CALL PP2DINT(KPROMA,KST,KND,KFLEVG,KPPM,KOPLEV,ILOLEV,&
       & IHILEV,ILEVB,ISLCT2,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZTT0,ZPPT)  
    ELSEIF (CDCONF /= 'M') THEN
      LL=.TRUE.
      IF (LL_RHO) THEN
        CALL PPT(KPROMA,KST,KND,KFLEVG,KOPLEV,1,KPPM,ILEVB,&
         & ZPRES,ZLNPRES,LLBELO,LLBELS,LLBLOW,LLBLES,LL,&
         & POROG,ZPXP,ZPXPD,ZTSTAR,ZT0,ZR,ZTT0,ZPPT,ZSTTF,&
         & PR2=ZR2,PRTPP=ZPPRT)
      ELSE
        CALL PPT(KPROMA,KST,KND,KFLEVG,KOPLEV,1,KPPM,ILEVB,&
         & ZPRES,ZLNPRES,LLBELO,LLBELS,LLBLOW,LLBLES,LL,&
         & POROG,ZPXP,ZPXPD,ZTSTAR,ZT0,ZR,ZTT0,ZPPT,ZSTTF)
      ENDIF   
    ELSE
      DO JLEVP=1,KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPT(KST:KND,JLEVP)=ZTT0(KST:KND,ILEVM)
      ENDDO
      IF (LL_RHO) THEN
        DO JLEVP=1,KOPLEV
          ILEVM=INT(PXLEV(JLEVP))
          ZPPRT(KST:KND,JLEVP)=ZR2(KST:KND,ILEVM)*ZTT0(KST:KND,ILEVM)
        ENDDO 
      ENDIF
    ENDIF
  ENDIF

!*      4.2.3    UPPER AIR PRESSURE :

  IF (LL_P.AND.CDCONF /= 'B') THEN
    IF (CDCONF /= 'M') THEN
      ZPPUAP(KST:KND,1:KOPLEV)=ZPRES(KST:KND,1:KOPLEV)
    ELSE
      DO JLEVP=1,KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPUAP(KST:KND,JLEVP)=ZPRESF(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

!*      4.2.4    SURFACE PRESSURE :

  IF ( LL_SP.OR.LL_LNSP ) THEN
    ZPPSP(KST:KND)=ZPRESH(KST:KND,KFLEVG)
  ENDIF

!*      4.2.5   PRESSURE DEPARTURE :

  IF (LL_PD.OR.LL_RHO) THEN
    IF (CDCONF /= 'M') THEN
      CALL PPQ(KPROMA,KST,KND,KFLEVG,KOPLEV,ILOLEV,KPPM,&
       & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZPDEP,ZPPPDEP)  
    ELSE
      DO JLEVP=1,KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPPDEP(KST:KND,JLEVP)=ZPDEP(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

!*      4.2.6   TRUE VERTICAL VELOCITY W :

  IF (LL_VW) THEN
    IF (CDCONF /= 'M') THEN
      CALL PP2DINT(KPROMA,KST,KND,KFLEVG,KPPM,KOPLEV,&
       & ILOLEV,IHILEV,ILEVB,ISLCT1,LLBELS,LLBLES,&
       & ZPRES,ZPXP,ZPXPD,ZWW,ZPPWVEL)  
    ELSE
      ! ZPPWVEL is filled with upper half level values
      DO JLEVP=1,KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPWVEL(KST:KND,JLEVP)=ZWW(KST:KND,ILEVM-1)
      ENDDO
    ENDIF
  ENDIF

!*      4.2.7   VERTICAL DIVERGENCE VARIABLE -G*(Delta w): 

  IF (LL_VD) THEN
    IF (CDCONF == 'S') THEN
      DO JH=0,1
        CALL PP2DINT(KPROMA,KST,KND,KFLEVG,KPPM,KOPLEV,ILOLEV,&
         & IHILEV,ILEVBH(1,1,1,JH),ISLCT1,&
         & LLBELSH(1,1,JH),LLBLESH(1,JH),ZPH(1,1,JH),&
         & ZPXP,ZPXPD,ZWW,ZPPWVELH(1,1,JH))  
      ENDDO
      DO JLEVP=1,KOPLEV
        DO JROF=KST,KND
          ZPPMGDW(JROF,JLEVP)=-RG*(ZPPWVELH(JROF,JLEVP,1)&
           & -ZPPWVELH(JROF,JLEVP,0))  
        ENDDO
      ENDDO
    ELSEIF (CDCONF /= 'M') THEN
      CALL ABOR1('VERT. DIVERGENCE IS COMPUTABLE ONLY ON ETA LEVELS')
    ELSE
      DO JLEVP=1,KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPMGDW(KST:KND,JLEVP)=ZMGDW(KST:KND,ILEVM)
      ENDDO
    ENDIF
  ENDIF

!*      4.2.8   HUMID DENSITY :

  IF (LL_RHO) THEN
    IF (CDCONF /= 'M') THEN
      DO JLEVP=1,KOPLEV
        ZPPRHO(KST:KND,JLEVP)=(ZPPPDEP(KST:KND,JLEVP)+ZPRES(KST:KND,JLEVP))/ &
          & ZPPRT(KST:KND,JLEVP)
      ENDDO
    ELSEIF (CDCONF == 'V') THEN
      CALL ABOR1('HUMID DENSITY IS NOT COMPUTED ON PV LEVELS')  
    ELSE
      DO JLEVP=1,KOPLEV
        ILEVM=INT(PXLEV(JLEVP))
        ZPPRHO(KST:KND,JLEVP)=(ZPPPDEP(KST:KND,JLEVP)+ZPRES(KST:KND,ILEVM))/ &
          & ZPPRT(KST:KND,JLEVP)
      ENDDO
    ENDIF
  ENDIF

ENDIF

!*    4.3   OTHER TRADITIONAL FIELDS

!*      4.3.1   GEOPOTENTIAL & MONTGOMERY POTENTIAL

IF (LL_Z .OR. LL_MG) THEN
  IF (CDCONF == 'B'.OR.CDCONF == 'T'.OR.CDCONF == 'S') THEN
    LLEXTR=.TRUE.
    CALL PPGEOP(KPROMA,KST,KND,KFLEVG,KOPLEV,ILOLEV,KPPM,ILEVB,ZPRES,ZLNPRES,&
     & LLBELO,LLBELS,LLBLOW,LLBLES,LLEXTR,POROG,&
     & ZPXP,ZPXPD,ZTSTAR,ZR,ZXYB(1,1,YYTXYBPP%M_LNPR),ZXYB(1,1,YYTXYBPP%M_ALPH),ZTT0,ZPPPHI,&
     & PSTTF=ZSTTF,PRRED=ZRRED)
  ELSEIF (CDCONF == 'V') THEN
    ! Do not use PPGEOP because it contains an implicit definition of
    ! the tropopause
    CALL PP2DINT(KPROMA,KST,KND,KFLEVG,KPPM,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT2,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZGEOPF,ZPPPHI)  
  ELSEIF (CDCONF == 'M') THEN
    DO JLEVP=1,KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPPHI(KST:KND,JLEVP)=ZGEOPF(KST:KND,ILEVM)
    ENDDO
  ELSEIF (CDCONF == 'K') THEN
    DO JLEVP=1,KOPLEV
      ZPPPHI(KST:KND,JLEVP)=ZGEOP(KST:KND,JLEVP)
    ENDDO
  ENDIF
  IF(LL_MG) THEN
    DO JLEVP=1,KOPLEV
      DO JROF=KST,KND
        ZPPMG(JROF,JLEVP)=ZPPPHI(JROF,JLEVP)+&
         & RCPD*PXLEV(JLEVP)*((ZPRES(JROF,JLEVP)/RATM)**RKAPPA)  
      ENDDO
    ENDDO
  ENDIF
ENDIF

!*      4.3.2   RELATIVE HUMIDITY :

IF (LL_HU) THEN
  IF (CDCONF /= 'M') THEN
    CALL PPQ(KPROMA,KST,KND,KFLEVG,KOPLEV,ILOLEV,KPPM,ILEVB,&
     & ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZRHF,ZPPHRL)
  ELSE
    DO JLEVP=1,KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPHRL(KST:KND,JLEVP)=ZRHF(KST:KND,ILEVM)
    ENDDO
  ENDIF
  IF (LFPRH100) THEN
    ! convert relative humidity field to percent
    DO JLEVP=1,KOPLEV
      DO JROF=KST,KND
        ZPPHRL(JROF,JLEVP)=ZPPHRL(JROF,JLEVP)*100.0_JPRB
      ENDDO
    ENDDO
  ENDIF
ENDIF

!*      4.3.3   VERTICAL VELOCITY "OMEGA":

IF (LL_VV) THEN
  IF (CDCONF /= 'M') THEN
    CALL PPVVEL(KPROMA,KST,KND,KFLEVG,KOPLEV,ILOLEV,KPPM,ILEVB,&
     & ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,&
     & ZXYB(1,1,YYTXYBPP%M_RDELP),ZCTY(1,0,YYTCTYPP%M_EVEL),ZCTY(1,1,YYTCTYPP%M_VVEL),ZPPVVE,.FALSE.)
  ELSE
    DO JLEVP=1,KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      DO JROF=KST,KND
        ZPPVVE(JROF,JLEVP)=ZPXP(JROF,ILEVM,2)*ZCTY(JROF,ILEVM,YYTCTYPP%M_VVEL)
      ENDDO
    ENDDO
  ENDIF
ENDIF

!*      4.3.4   ETADOT

IF (LL_ETAD) THEN
  IF (CDCONF /= 'M') THEN
    CALL PPVVEL(KPROMA,KST,KND,KFLEVG,KOPLEV,ILOLEV,KPPM,ILEVB,&
     & ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,&
     & ZXYB(1,1,YYTXYBPP%M_RDELP),ZCTY(1,0,YYTCTYPP%M_EVEL),ZCTY(1,1,YYTCTYPP%M_VVEL),ZPPEVEL,.TRUE.)
  ELSE
    DO JLEVP=1,KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPEVEL(KST:KND,JLEVP)=ZETADOT(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      4.3.5   MSL PRESSURE :

IF (LL_MSL) THEN
  CALL PPPMER(KPROMA,KST,KND,ZPRESH(1,KFLEVG),POROG,ZTSTAR,ZT0,ZPPMSLP)
ENDIF

!*      4.3.6   Filtred ln(Ps) for monitoring Couplinf Update Frequency :

DO JCUFNR=1,NCUFNR
  IF (LL_CUF(JCUFNR)) ZPPCUF(KST:KND,JCUFNR)=PRMCUFGP(KST:KND,JCUFNR)
ENDDO

!*    4.4   TRADITIONAL FIELDS FOR CONFIGURATIONS CALLING 'APACHE'

IF (LLAP) THEN

  CALL FPPS(KPROMA,KST,KND,KFLEVG,1,ZXGEOPH,POROG,ZTT0(1,1),&
   & ZR,ZPRESH,PTSI,ZPPSP)

  IF (CDCONF == 'S') THEN
    IF (LRWSDLR) THEN
      ! ky: call to GPHPRE in PPLETA not designed for LRWSDLR=T
      CALL ABOR1(' POS: PPLETA not properly updated for LRWSDLR')
    ENDIF
    ! recompute ZPPUAP, ZPH
    DO JLEVP=1,KOPLEV
      ZXLEV(JLEVP)=REAL(JLEVP,JPRB)
    ENDDO
    CALL PPLETA(KPROMA,KST,KND,KOPLEV,KOPLEV,ZXLEV,&
     & ZPPSP,ZPPUAP,ZPH(1,1,1),ZPH(1,1,0),PVAH=PVAH,PVBH=PVBH)
  ENDIF

  ! APACHE returns ZPPSP=surface pressure (NOT log surface pressure)
  ! (of course, like POS itself)
  ! ZPDT0 contains pre-prehyd because APACHE is called there if LFPART2 only.
  CALL APACHE(KPROMA,KST,KND,KFLEVG,KOPLEV,IPPGFL,KPPM,YLPPIC,&
   & LL_PD,LL_VD,LL_VW,LLAP_GFL,.FALSE.,LFPQ,ZRHBNDS,&
   & PXORO,POROG,FPBL,PTSI,PXTSO,ZPPSP,ZPPUAP,ZPH,&
   & ZTT0,ZUT0,ZVT0,ZPDT0,ZWW,ZGFLT0,&
   & ZPRESH,ZPRESF,ZXYB(1,1,YYTXYBPP%M_LNPR),ZXYB(1,1,YYTXYBPP%M_ALPH),ZR,ZKAP,ZRHF,&
   & ZPPT,ZPPU,ZPPV,ZPPPDEP,ZPPWVELH,ZPPWVEL,ZPPGFL,ZPPPHI,ZPPHRL,ZDUMARR)

  IF (LL_VD) THEN
    DO JLEVP=1,KOPLEV
      DO JROF=KST,KND
        ZPPMGDW(JROF,JLEVP)=-RG*(ZPPWVELH(JROF,JLEVP,1)&
         & -ZPPWVELH(JROF,JLEVP,0))  
      ENDDO
    ENDDO
  ENDIF

ENDIF

!*    4.5   MODERN DYNAMICAL METEOROLOGY FIELDS

!*      4.5.1   POTENTIAL VORTICITY

IF (LL_PV.AND.CDCONF /= 'V') THEN
  IF (CDCONF /= 'M') THEN
    CALL PP2DINT(KPROMA,KST,KND,KFLEVG,KPPM,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT2,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZTOUP,ZPPPOTV)  
  ELSE
    DO JLEVP=1,KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPPOTV(KST:KND,JLEVP)=ZTOUP(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      4.5.2   POTENTIAL TEMPERATURE

IF (LL_TH.AND.CDCONF /= 'T') THEN
  IF (CDCONF /= 'M') THEN
    CALL PP2DINT(KPROMA,KST,KND,KFLEVG,KPPM,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT2,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZTETA,ZPPTETA)  
  ELSE
    DO JLEVP=1,KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPTETA(KST:KND,JLEVP)=ZTETA(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      4.5.3   EQUIVALENT POTENTIAL TEMPERATURE

IF (LL_ETH) THEN
  IF (CDCONF /= 'M') THEN
    CALL PP2DINT(KPROMA,KST,KND,KFLEVG,KPPM,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT2,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZEPTH,ZPPEPTH)  
  ELSE
    DO JLEVP=1,KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPEPTH(KST:KND,JLEVP)=ZEPTH(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*     4.5.3.1
IF (LL_VTG) THEN
 IF (CDCONF /= 'M') THEN
    CALL PP2DINT (KPROMA,KST,KND,KFLEVG,KPPM,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT2,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZVTGH,ZPPVTG)
  ELSE
    DO JLEVP = 1 , KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPVTG(KST:KND,JLEVP)=ZVTGF(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF


!*      4.5.4   STRETCHING DEFORMATION

IF (LL_STD) THEN
  IF (CDCONF /= 'M') THEN
    CALL PP2DINT(KPROMA,KST,KND,KFLEVG,KPPM,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT2,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZSTDF,ZPPSTDF)  
  ELSE
    DO JLEVP=1,KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPSTDF(KST:KND,JLEVP)=ZSTDF(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      4.5.5   SHEARING DEFORMATION

IF (LL_SHD) THEN
  IF (CDCONF /= 'M') THEN
    CALL PP2DINT(KPROMA,KST,KND,KFLEVG,KPPM,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT2,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZSHDF,ZPPSHDF)  
  ELSE
    DO JLEVP=1,KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPSHDF(KST:KND,JLEVP)=ZSHDF(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      4.5.6   TOTAL WATER VAPOUR

IF (LL_TWV) THEN
  Z1SRG=1.0_JPRB/RG
  DO JLEV=1,KFLEVG
    DO JROF=KST,KND
      ZPPTWV(JROF)=ZPPTWV(JROF)+ZGFLT0(JROF,JLEV,YLPPIC%IQ)*ZXYB(JROF,JLEV,YYTXYBPP%M_DELP)*Z1SRG
    ENDDO
  ENDDO
ENDIF

!*      4.5.7   TROPOPAUSE FOLDING INDICATOR

IF (LL_FOL) THEN
  ZTROPO=2.E-6_JPRB ! THIS IS ANOTHER INTERNAL DEFINITION OF THE TROPOPAUSE !
  CALL PPLTP(KPROMA,KST,KND,KFLEVG,ZPRESF,ZTOUP(1,1),ZTROPO,.TRUE.,PRCORI,&
   & ZWORK,ZLNWORK,ZPPFOLDIN)  
ENDIF

!*      4.5.8   ISOBARIC EQUIVALENT TEMPERATURE

IF (LL_IET) THEN
  IF (CDCONF /= 'M') THEN
    CALL PP2DINT(KPROMA,KST,KND,KFLEVG,KPPM,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT2,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZIBET,ZPPIBET)  
  ELSE
    DO JLEVP=1,KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPIBET(KST:KND,JLEVP)=ZIBET(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      4.5.9   SIMULATED REFLECTIVITIES

IF (LL_SRE) THEN
  IF (CDCONF /= 'M') THEN
    CALL PP2DINT(KPROMA,KST,KND,KFLEVG,KPPM,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT2,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZSIMR,ZPPSIMR)  
  ELSE
    DO JLEVP=1,KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPSIMR(KST:KND,JLEVP)=ZSIMR(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      4.5.10  THETAv

IF (LL_THV) THEN
  IF (CDCONF /= 'M') THEN
    CALL PP2DINT(KPROMA,KST,KND,KFLEVG,KPPM,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT2,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZVIRT,ZPPVIRT)  
  ELSE
    DO JLEVP=1,KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPVIRT(KST:KND,JLEVP)=ZVIRT(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*    4.6   OTHER FIELDS

!*      4.6.1   MOIST IRREVERSIBLE ADIABATIC POTENTIAL TEMPERATURE

IF (LL_THPW) THEN
  I=TFP_THPW%ICOD
  DO JL=1,QFPTYPE%ILEV(I)
    ILPT=QFPTYPE%ILVP(JL,I)
    IF (CDCONF /= 'M') THEN
      CALL PPTHPW(KST,KND,KPROMA,ZPRES(1,ILPT),ZPPT(1,ILPT),ZPPGFL(1,ILPT,YLPPIC%IQ),&
       & ZPPGFL(1,ILPT,YLPPIC%IL),ZPPGFL(1,ILPT,YLPPIC%II),ZPPTHPW(1,ILPT))  
    ELSE
      ILEVM=INT(PXLEV(ILPT))
      CALL PPTHPW(KST,KND,KPROMA,ZPRESF(1,ILEVM),ZPPT(1,ILPT),&
       & ZPPGFL(1,ILPT,YLPPIC%IQ),ZPPGFL(1,ILPT,YLPPIC%IL),&
       & ZPPGFL(1,ILPT,YLPPIC%II),ZPPTHPW(1,ILPT))  
    ENDIF
  ENDDO
ENDIF

!*      4.6.2   WIND VELOCITY

IF (LL_WND) THEN
  DO JLEVP=1,KOPLEV
    DO JROF=KST,KND
      ZPPWIND(JROF,JLEVP)=SQRT( &
       & ZPPU(JROF,JLEVP)*ZPPU(JROF,JLEVP)+ZPPV(JROF,JLEVP)*ZPPV(JROF,JLEVP) )
    ENDDO
  ENDDO
ENDIF

!*      4.6.3   DIVERGENCE

IF (LL_DIV) THEN
  IF (CDCONF /= 'M') THEN
    CALL PP2DINT(KPROMA,KST,KND,KFLEVG,KPPM,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT2,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZDIVT0,ZPPDIV)  
  ELSE
    DO JLEVP=1,KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPDIV(KST:KND,JLEVP)=ZDIVT0(KST:KND,ILEVM)
    ENDDO
  ENDIF
ENDIF

!*      4.6.4   VORTICITY

IF (LL_VOR.OR.LL_ABS) THEN
  IF (CDCONF /= 'M') THEN
    CALL PP2DINT(KPROMA,KST,KND,KFLEVG,KPPM,KOPLEV,ILOLEV,&
     & IHILEV,ILEVB,ISLCT2,LLBELO,LLBLOW,ZPRES,ZPXP,ZPXPD,ZVORT0,ZPPVOR)  
  ELSE
    DO JLEVP=1,KOPLEV
      ILEVM=INT(PXLEV(JLEVP))
      ZPPVOR(KST:KND,JLEVP)=ZVORT0(KST:KND,ILEVM)
    ENDDO
  ENDIF
  IF (LL_ABS) THEN
    IF (CFPFMT == 'MODEL') THEN
      DO JLEVP=1,KOPLEV
        ZPPABS(KST:KND,JLEVP)=ZPPVOR(KST:KND,JLEVP)+PRCORI(KST:KND)
      ENDDO
    ELSE
      ! Coriolis parameter will be added during horizontal interpolations
      DO JLEVP=1,KOPLEV
        ZPPABS(KST:KND,JLEVP)=ZPPVOR(KST:KND,JLEVP)
      ENDDO
    ENDIF
  ENDIF
ENDIF

!*      4.6.5   ICAO FIELDS

IF (LLAERO) THEN
  IPOL2=4
  CALL POAERO(ZGEOPF(1,1),ZTT0(1,1),ZUT0(1,1),ZVT0(1,1),&
   & ZPPPICAO,ZPPTICAO,ZPPUJET,ZPPVJET,ZPPPJET,&
   & ZPRESF,KPROMA,KST,KND,KFLEVG,IPOL2)
ENDIF

!*      4.6.6 ISO-TEMPERATURE FIELDS

IF (LL_PTB .OR. LL_HTB .AND. CDCONF=='K') THEN
  IF (LL_PTB) THEN
    ZPPPTB(KST:KND,1:KOPLEV)=ZPRES(KST:KND,1:KOPLEV)  
  ENDIF
  IF (LL_HTB) THEN
    ! Convert from geopotentials to meters
    Z1SRG=1.0_JPRB/RG
    ZPPHTB(KST:KND,1:KOPLEV)=ZGEOP(KST:KND,1:KOPLEV)*Z1SRG
  ENDIF
ENDIF

IF (LL_HTPW) THEN
  DO JLEV=1,KFLEVG
    CALL PPWETPOINT(KST,KND,KPROMA,ZPRESF(1,JLEV),ZTT0(1,JLEV),ZGFLT0(1,JLEV,YLPPIC%IQ),&
     & ZGFLT0(1,JLEV,YLPPIC%IL),ZGFLT0(1,JLEV,YLPPIC%II),ZTPW(1,JLEV))  
  ENDDO
  LLTOP=.FALSE.
  ZXTEMP=RTT
  CALL PPLTEMP(KPROMA,KST,KND,KFLEVG,ZGEOPF(1,1),ZTPW,ZXTEMP,LLTOP,ZPPHTPW)
  ! Convert from geopotentials to meters
  Z1SRG=1.0_JPRB/RG
  ZPPHTPW(KST:KND)=ZPPHTPW(KST:KND)*Z1SRG
ENDIF

!*      4.6.7   MOISTURE CONVERGENCE :

IF (LL_MOCO) THEN
!  CALL MOCONI(KST,KND,ZPRESH(1,KFLEVG),ZPRESF,ZXYB(1,1,YYTXYBPP%M_DELP),ZUT0(1,1),ZVT0(1,1),&
!   &ZDIVT0(1,1),ZGFLT0(1,1,YLPPIC%IQ),ZQT0L,ZQT0M,ZGEOPF(1,1),ZGEOPH,ZR,ZTT0(1,1),&
!   &ZTSTAR,PGCLP,ZPPMOCO)
ENDIF

!*      4.6.8   QNH PRESSURE :

IF (LL_QNH) THEN
  CALL PPPMER(KPROMA,KST,KND,ZPRESH(1,KFLEVG),POROG,ZTSTAR,ZT0,ZPPQNH,&
   & LDQNH=.TRUE.)
ENDIF

!*    4.7   FREE-USE FIELDS

IF (ANY(LL_FUA)) THEN

  ! provide vertical post-processing here in pressure with result ZPPFUA
  ! in general PPQ is used for interpolation in the vertical in pressure

  IF (YSD_XAD%NUMFLDS > 0) THEN
    IF(YSD_XAD%NLEVS /= KFLEVG) THEN
      CALL ABOR1('POS: YSD_XAD%NLEVS /= KFLEVG not allowed with LFPOS=.T.')
    ENDIF
    IF(YSD_XAD%NUMFLDS > JPOSVX2) THEN
      CALL ABOR1('POS: JPOSVX2 too small for YSD_XAD%NUMFLDS')
    ENDIF
  ENDIF
  IPTR=1
  DO JVAR=1,JPOSVX2
    IF (LL_FUA(JVAR)) THEN
      DO JVEXTR=1,YSD_XAD%NUMFLDS
        IF (TFP_FUA(JVAR)%IGRIB == YSD_XA%YXA(JVEXTR)%IGRBCODE(1)) THEN
          DO JLEV=1,YSD_XAD%NLEVS
            ZEXTRA(KST:KND,JLEV)=PEXTRA(KST:KND,JLEV,JVEXTR)
          ENDDO
          CALL PPQ(KPROMA,KST,KND,KFLEVG,KOPLEV,ILOLEV,KPPM,&
           & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZEXTRA,ZPPFUA(1,1,IPTR))  
          IPTR=IPTR+1
          EXIT
        ENDIF
      ENDDO
    ENDIF
  ENDDO

  IF (YSD_XRD%NUMFLDS > 0) THEN
    IF(YSD_XRD%NLEVS /= KFLEVG) THEN
      CALL ABOR1 ('POS: YSD_XRD%NLEVS /= KFLEVG not allowed with LFPOS=.T.')
    ENDIF
    IF(YSD_XRD%NUMFLDS > JPOSVX2) THEN
      CALL ABOR1 ('POS: JPOSVX2 too small for YSD_XRD%NUMFLDS')
    ENDIF
  ENDIF
  IPTR=1
  DO JVAR=1,JPOSVX2
    IF (LL_FUA(JVAR)) THEN
      DO JVEXTR=1,YSD_XRD%NUMFLDS
        IF (TFP_FUA(JVAR)%IGRIB == YSD_XR%YXA(JVEXTR)%IGRBCODE(1)) THEN
          DO JLEV = 1 , YSD_XRD%NLEVS
            ZEXRAD(KST:KND,JLEV) = PEXRAD(KST:KND,JLEV,JVEXTR)
          ENDDO
          CALL PPQ(KPROMA,KST,KND,KFLEVG,KOPLEV,ILOLEV,KPPM,&
           & ILEVB,ZPRES,LLBELO,LLBLOW,ZPXP,ZPXPD,ZEXRAD,ZPPFUA(1,1,IPTR))
          IPTR=IPTR+1
          EXIT
        ENDIF
      ENDDO
    ENDIF
  ENDDO

ENDIF

IF (ANY(LL_FSU)) THEN
ENDIF

!     ------------------------------------------------------------------

!*       5.     STORE POST-PROCESSED FIELDS.
!               ----------------------------

!*    5.1   CONSTANT QUANTITIES (orography, mapping factor)

I=TFP_FIS%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,POROG,PGPP,PAUX)

I=TFP_GM%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,PGM,PGPP,PAUX)

!*    5.2   TRADITIONAL GFL FIELDS

DO JVAR=1,IPPGFL
  I=IGFLCOD(JVAR)
  IF (QFPTYPE%LL(I)) THEN
    ZPPIN(KST:KND,1:KOPLEV)=ZPPGFL(KST:KND,1:KOPLEV,JVAR)
    CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPIN,PGPP,PAUX)
  ENDIF
ENDDO

!*    5.3   TRADITIONAL GMV+GMVS FIELDS

I=TFP_U%ICOD
IF (QFPTYPE%LL(I)) THEN
  DO JLEVP=1,KOPLEV
    ZPPIN(KST:KND,JLEVP)=ZPPU(KST:KND,JLEVP)/PGM(KST:KND)
  ENDDO
  CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPIN,PGPP,PAUX)
ENDIF

I=TFP_V%ICOD
IF (QFPTYPE%LL(I)) THEN
  DO JLEVP=1,KOPLEV
    ZPPIN(KST:KND,JLEVP)=ZPPV(KST:KND,JLEVP)/PGM(KST:KND)
  ENDDO
  CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPIN,PGPP,PAUX)
ENDIF

I=TFP_T%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPT,PGPP,PAUX)

I=TFP_P%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPUAP,PGPP,PAUX)

IF ((LFPLOSP.AND.QFPTYPE%LL(TFP_SP%ICOD)).OR.QFPTYPE%LL(TFP_LNSP%ICOD)) THEN
  DO JROF=KST,KND
    ZPPLNSP(JROF)=LOG(ZPPSP(JROF))
  ENDDO
ENDIF
I=TFP_SP%ICOD
IF (LFPLOSP) THEN
  IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,ZPPLNSP,PGPP,PAUX)
ELSE
  IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,ZPPSP,PGPP,PAUX)
ENDIF
I=TFP_LNSP%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,ZPPLNSP,PGPP,PAUX)

I=TFP_PD%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPPDEP,PGPP,PAUX)

I=TFP_VW%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPWVEL,PGPP,PAUX)

I=TFP_WWS%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,ZWW(1,KFLEVG),PGPP,PAUX)

I=TFP_VD%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPMGDW,PGPP,PAUX)

!*    5.4   OTHER TRADITIONAL FIELDS

I=TFP_Z%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPPHI,PGPP,PAUX)

I=TFP_MG%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPMG,PGPP,PAUX)

I=TFP_HU%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPHRL,PGPP,PAUX)

I=TFP_VV%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPVVE,PGPP,PAUX)

I=TFP_ETAD%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPEVEL,PGPP,PAUX)

I=TFP_MSL%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,ZPPMSLP,PGPP,PAUX)

DO JCUFNR=1,NCUFNR
  I=ICUFCOD(JCUFNR)
  IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,ZPPCUF(1,JCUFNR),PGPP,PAUX)
ENDDO

!*    5.5   MODERN DYNAMICAL METEOROLOGY FIELDS

I=TFP_PV%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPPOTV,PGPP,PAUX)

I=TFP_TH%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPTETA,PGPP,PAUX)

I=TFP_VTG%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPVTG,PGPP,PAUX)

I=TFP_ETH%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPEPTH,PGPP,PAUX)

I=TFP_STD%ICOD
IF (QFPTYPE%LL(I)) THEN
  DO JLEVP=1,KOPLEV
    ZPPIN(KST:KND,JLEVP)=ZPPSTDF(KST:KND,JLEVP)/ZGMAPP(KST:KND)
  ENDDO
  CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPIN,PGPP,PAUX)
ENDIF

I=TFP_SHD%ICOD
IF (QFPTYPE%LL(I)) THEN
  DO JLEVP=1,KOPLEV
    ZPPIN(KST:KND,JLEVP)=ZPPSHDF(KST:KND,JLEVP)/ZGMAPP(KST:KND)
  ENDDO
  CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPIN,PGPP,PAUX)
ENDIF

I=TFP_TWV%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,ZPPTWV,PGPP,PAUX)

I=TFP_FOL%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,ZPPFOLDIN,PGPP,PAUX)

I=TFP_IET%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPIBET,PGPP,PAUX)

I=TFP_SRE%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPSIMR,PGPP,PAUX)

I=TFP_THV%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPVIRT,PGPP,PAUX)

!*    5.6   OTHER FIELDS

I=TFP_THPW%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPTHPW,PGPP,PAUX)

I=TFP_WND%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPWIND,PGPP,PAUX)

I=TFP_DIV%ICOD
IF (QFPTYPE%LL(I)) THEN
  IF (QFPTYPE%ISKP(I) == 2) THEN
    IF (QFPTYPE%ISF(I) >= 1) THEN
      DO JL=1,QFPTYPE%ILEV(I)
        ILPT=QFPTYPE%ILVP(JL,I)
        ILOCU=QFPTYPE%IGT1(I,1)+JL-1
        ILOCV=QFPTYPE%IGT1(I,2)+JL-1
        PGPP(KST:KND,ILOCU)=ZPPU(KST:KND,ILPT)/PGM(KST:KND)
        PGPP(KST:KND,ILOCV)=ZPPV(KST:KND,ILPT)/PGM(KST:KND)
      ENDDO
    ELSE
      CALL ABOR1('POS : INTERNAL ERROR ON DIV')
    ENDIF
  ELSE
    DO JLEVP=1,KOPLEV
      ZPPIN(KST:KND,JLEVP)=ZPPDIV(KST:KND,JLEVP)/ZGMAPP(KST:KND)
    ENDDO
    CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPIN,PGPP,PAUX)
  ENDIF
ENDIF

I=TFP_VOR%ICOD
IF (QFPTYPE%LL(I)) THEN
  IF (QFPTYPE%ISKP(I) == 2) THEN
    IF (QFPTYPE%ISF(I) >= 1) THEN
      DO JL=1,QFPTYPE%ILEV(I)
        ILPT=QFPTYPE%ILVP(JL,I)
        ILOCU=QFPTYPE%IGT1(I,1)+JL-1
        ILOCV=QFPTYPE%IGT1(I,2)+JL-1
        PGPP(KST:KND,ILOCU)=ZPPU(KST:KND,ILPT)/PGM(KST:KND)
        PGPP(KST:KND,ILOCV)=ZPPV(KST:KND,ILPT)/PGM(KST:KND)
      ENDDO
    ELSE
      CALL ABOR1('POS : INTERNAL ERROR ON DIV')
    ENDIF
  ELSE
    DO JLEVP=1,KOPLEV
      ZPPIN(KST:KND,JLEVP)=ZPPVOR(KST:KND,JLEVP)/ZGMAPP(KST:KND)
    ENDDO
    CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPIN,PGPP,PAUX)
  ENDIF
ENDIF

I=TFP_ABS%ICOD
IF (QFPTYPE%LL(I)) THEN
  IF (QFPTYPE%ISKP(I) == 2) THEN
    IF (QFPTYPE%ISF(I) >= 1) THEN
      DO JL=1,QFPTYPE%ILEV(I)
        ILPT=QFPTYPE%ILVP(JL,I)
        ILOCU=QFPTYPE%IGT1(I,1)+JL-1
        ILOCV=QFPTYPE%IGT1(I,2)+JL-1
        PGPP(KST:KND,ILOCU)=ZPPU(KST:KND,ILPT)/PGM(KST:KND)
        PGPP(KST:KND,ILOCV)=ZPPV(KST:KND,ILPT)/PGM(KST:KND)
      ENDDO
    ELSE
      CALL ABOR1('POS : INTERNAL ERROR ON DIV')
    ENDIF
  ELSE
    DO JLEVP=1,KOPLEV
      ZPPIN(KST:KND,JLEVP)=ZPPABS(KST:KND,JLEVP)/ZGMAPP(KST:KND)
    ENDDO
    CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPIN,PGPP,PAUX)
  ENDIF
ENDIF

I=TFP_PSI%ICOD
IF (QFPTYPE%LL(I)) THEN
  DO JL=1,QFPTYPE%ILEV(I)
    ILPT=QFPTYPE%ILVP(JL,I)
    ILOC=QFPTYPE%IGT1(I,1)+JL-1
    PGPP(KST:KND,ILOC)=ZPPV(KST:KND,ILPT)/PGM(KST:KND)
  ENDDO
ENDIF

I=TFP_KHI%ICOD
IF (QFPTYPE%LL(I)) THEN
  DO JL=1,QFPTYPE%ILEV(I)
    ILPT=QFPTYPE%ILVP(JL,I)
    ILOC=QFPTYPE%IGT1(I,1)+JL-1
    PGPP(KST:KND,ILOC)=ZPPU(KST:KND,ILPT)/PGM(KST:KND)
  ENDDO
ENDIF

I=TFP_PTB%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPPTB,PGPP,PAUX)

I=TFP_HTB%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPHTB,PGPP,PAUX)

I=TFP_RHO%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPRHO,PGPP,PAUX)

!---------------------------------
! surface 2D fields

I=TFP_PCAO%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,ZPPPICAO,PGPP,PAUX)

I=TFP_TCAO%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,ZPPTICAO,PGPP,PAUX)

I=TFP_UJET%ICOD
IF (QFPTYPE%LL(I)) THEN
  ZPPIN1(KST:KND)=ZPPUJET(KST:KND)/PGM(KST:KND)
  CALL FILL_PP2(KPROMA,KST,KND,I,ZPPIN1,PGPP,PAUX)
ENDIF

I=TFP_VJET%ICOD
IF (QFPTYPE%LL(I)) THEN
  ZPPIN1(KST:KND)=ZPPVJET(KST:KND)/PGM(KST:KND)
  CALL FILL_PP2(KPROMA,KST,KND,I,ZPPIN1,PGPP,PAUX)
ENDIF

I=TFP_PJET%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,ZPPPJET,PGPP,PAUX)

I=TFP_HTPW%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,ZPPHTPW,PGPP,PAUX)

I=TFP_MOCO%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,ZPPMOCO,PGPP,PAUX)

I=TFP_QNH%ICOD
IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,ZPPQNH,PGPP,PAUX)

!*    5.7   FREE-USE FIELDS

IPTR=1
DO JVAR=1,JPOSVX2
  I=TFP_FUA(JVAR)%ICOD
  IF (QFPTYPE%LL(I)) THEN
    CALL FILL_PP3(KPROMA,KST,KND,KOPLEV,I,ZPPFUA(1,1,IPTR),PGPP,PAUX)
    IPTR=IPTR+1
  ENDIF
ENDDO

DO JVAR=1,JPOSFSU
  I=TFP_FSU(JVAR)%ICOD
  IF (QFPTYPE%LL(I)) CALL FILL_PP2(KPROMA,KST,KND,I,ZPPFSU(1,JVAR),PGPP,PAUX)
ENDDO

!*    5.8   DEALLOCATIONS

DEALLOCATE(ZPPFUA)
DEALLOCATE(ZPPFSU)
IF(YSD_XAD%NUMFLDS > 0) THEN
  DEALLOCATE(ZEXTRA)
ENDIF
IF(YSD_XRD%NUMFLDS > 0) THEN
  DEALLOCATE(ZEXRAD)
ENDIF
IF (ALLOCATED(LLIN_GFL)) DEALLOCATE(LLIN_GFL)
IF (ALLOCATED(LL_GFL)) DEALLOCATE(LL_GFL)
IF (ALLOCATED(LLAP_GFL)) DEALLOCATE(LLAP_GFL)
IF (ALLOCATED(IGFLCOD)) DEALLOCATE(IGFLCOD)
IF (ALLOCATED(IDIN_GFL)) DEALLOCATE(IDIN_GFL)
IF (ALLOCATED(ZPPGFL)) DEALLOCATE(ZPPGFL)
IF (ALLOCATED(ZGFLT0)) DEALLOCATE(ZGFLT0)

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('POS',1,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

CONTAINS

SUBROUTINE FILL_PP2(KPROMA,KST,KND,KCOD,PIN,PGPP,PAUX)

INTEGER(KIND=JPIM), INTENT(IN)  :: KPROMA
INTEGER(KIND=JPIM), INTENT(IN)  :: KST
INTEGER(KIND=JPIM), INTENT(IN)  :: KND
INTEGER(KIND=JPIM), INTENT(IN)  :: KCOD
REAL(KIND=JPRB)   , INTENT(IN)  :: PIN(KPROMA)
REAL(KIND=JPRB)   , INTENT(OUT) :: PGPP(KPROMA,KFLDSPP)
REAL(KIND=JPRB)   , INTENT(OUT) :: PAUX(KPROMA,KFLDAUX)

INTEGER(KIND=JPIM) :: ILOC
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('POS:FILL_PP2',0,ZHOOK_HANDLE)

IF (QFPTYPE%ISF(KCOD) >= 1) THEN
  ILOC=QFPTYPE%IGT1(KCOD,1)
  PGPP(KST:KND,ILOC) = PIN(KST:KND)
ELSE
  ILOC=QFPTYPE%IGPX(KCOD)
  PAUX(KST:KND,ILOC) = PIN(KST:KND)
ENDIF

IF (LHOOK) CALL DR_HOOK('POS:FILL_PP2',1,ZHOOK_HANDLE)

END SUBROUTINE FILL_PP2

!     ------------------------------------------------------------------

SUBROUTINE FILL_PP3(KPROMA,KST,KND,KOPLEV,KCOD,PIN,PGPP,PAUX)

INTEGER(KIND=JPIM), INTENT(IN)  :: KPROMA
INTEGER(KIND=JPIM), INTENT(IN)  :: KST
INTEGER(KIND=JPIM), INTENT(IN)  :: KND
INTEGER(KIND=JPIM), INTENT(IN)  :: KOPLEV
INTEGER(KIND=JPIM), INTENT(IN)  :: KCOD
REAL(KIND=JPRB)   , INTENT(IN)  :: PIN(KPROMA,KOPLEV)
REAL(KIND=JPRB)   , INTENT(OUT) :: PGPP(KPROMA,KFLDSPP)
REAL(KIND=JPRB)   , INTENT(OUT) :: PAUX(KPROMA,KFLDAUX)

INTEGER(KIND=JPIM) :: ILOC,ILPT,JL
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('POS:FILL_PP3',0,ZHOOK_HANDLE)

DO JL=1,QFPTYPE%ILEV(KCOD)
  ILPT=QFPTYPE%ILVP(JL,KCOD)
  IF (QFPTYPE%ISF(KCOD) >= 1) THEN
    ILOC=QFPTYPE%IGT1(KCOD,1)+JL-1
    PGPP(KST:KND,ILOC) = PIN(KST:KND,ILPT)
  ELSE
    ILOC=QFPTYPE%IGPX(KCOD)+JL-1
    PAUX(KST:KND,ILOC) = PIN(KST:KND,ILPT)
  ENDIF
ENDDO

IF (LHOOK) CALL DR_HOOK('POS:FILL_PP3',1,ZHOOK_HANDLE)

END SUBROUTINE FILL_PP3

!     ------------------------------------------------------------------

SUBROUTINE SUPTRPPGFL_POS(KFL,KPPGFL,KMPP)
! computes pointers to store interpolated GFL variables.

INTEGER(KIND=JPIM), INTENT(IN)     :: KFL ! number of fields
INTEGER(KIND=JPIM), INTENT(INOUT)  :: KPPGFL ! cumulated number of fields
INTEGER(KIND=JPIM), INTENT(OUT)    :: KMPP ! pointer

REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('POS:SUPTRPPGFL_POS',0,ZHOOK_HANDLE)

KMPP=KPPGFL+1
KPPGFL=KPPGFL+KFL

IF (LHOOK) CALL DR_HOOK('POS:SUPTRPPGFL_POS',1,ZHOOK_HANDLE)

END SUBROUTINE SUPTRPPGFL_POS

!     ------------------------------------------------------------------
END SUBROUTINE POS
